#!/usr/bin/env python
# -*- coding: utf-8 -*-

from nose.tools import *
import hwrt


def setup():
    print("SETUP!")


def teardown():
    print("TEAR DOWN!")


def test_basic():
    print("I RAN!")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# hwrt modules
import hwrt.download as download


# Tests
def execution_test():
    download.get_parser()
    # download.is_file_consistent(31, 'mysql_online')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from nose.plugins.skip import SkipTest
import shutil
import os

# hwrt modules
import hwrt.view as view
import hwrt.utils as utils


# Tests
def execution_test():
    view.get_parser()
    view._fetch_data_from_server(31, 'mysql_online')

    d = os.path.dirname(__file__)
    target = os.path.join(utils.get_project_root(),
                          'raw-datasets/unittests-tiny-raw.pickle')
    shutil.copyfile(os.path.join(d, 'data/unittests-tiny-raw.pickle'),
                    target)
    view._get_data_from_rawfile(target, 345)  # Is in tiny test set
    view._get_data_from_rawfile(target, 42)  # Is not in tiny test set
    view._list_ids(target)
    model_folder = os.path.join(utils.get_project_root(),
                                'models/small-baseline')
    view._get_system(model_folder)
    #display_data(raw_data_string, raw_data_id, model_folder, show_raw)


@SkipTest
def execute_main_test():
    model_small = os.path.join(utils.get_project_root(),
                               "models",
                               "small-baseline")
    view.main(True, model_small, False, 31, False, 'mysql_online')
    view.main(False, model_small, False, 31, False, 'mysql_online')
    view.main(False, model_small, True, 31, False, 'mysql_online')
    view.main(False, model_small, False, 31, True, 'mysql_online')

    # with patch('sys.exit') as exit_mock:
    #     view._get_description('.')
    #     assert exit_mock.called



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import nose
import tests.testhelper as testhelper

# hwrt modules
from hwrt.HandwrittenData import HandwrittenData
import hwrt.preprocessing as preprocessing
import hwrt.features as features


# Tests
def feature_detection_test():
    l = [{'StrokeCount': None},
         {'ConstantPointCoordinates':
          [{'strokes': 4},
           {'points_per_stroke': 81},
           {'fill_empty_with': 0},
           {'pen_down': False}]
          }
         ]
    correct = [features.StrokeCount(),
               features.ConstantPointCoordinates(strokes=4,
                                                 points_per_stroke=81,
                                                 fill_empty_with=0,
                                                 pen_down=False)]
    feature_list = features.get_features(l)
    # TODO: Not only compare lengths of lists but actual contents.
    nose.tools.assert_equal(len(feature_list), len(correct))


def repr_and_str_test():
    l = [features.ConstantPointCoordinates(),
         features.FirstNPoints(),
         features.StrokeCount(),
         features.Ink()
         ]
    for alg in l:
        str(alg)
        repr(alg)


def dimension_test():
    l = [(features.ConstantPointCoordinates(), 160),
         (features.FirstNPoints(), 162),  # TODO: Check
         (features.StrokeCount(), 1),
         (features.Ink(), 1)
         ]
    for alg, dimension in l:
        nose.tools.assert_equal(alg.get_dimension(), dimension)


def height_test():
    feature_list = [features.Height()]
    a = testhelper.get_symbol_as_handwriting(97705)
    # TODO: Check if this is correct
    nose.tools.assert_equal(a.feature_extraction(feature_list), [263.0])


def stroke_count_test():
    feature_list = [features.StrokeCount()]
    a = testhelper.get_symbol_as_handwriting(97705)
    nose.tools.assert_equal(a.feature_extraction(feature_list), [1])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import nose
import os
import shutil

# hwrt modules
import hwrt.preprocess_dataset as preprocess_dataset
import hwrt.utils as utils
import hwrt.preprocessing as preprocessing


# Tests
@nose.tools.nottest
def execution_test():
    # TODO: nose.proxy.UnicodeDecodeError: 'utf-8' codec can't decode byte 0x80
    #                                      in position 0: invalid start byte
    small = os.path.join(utils.get_project_root(),
                         "preprocessed/small-baseline")
    preprocess_dataset.main(small)


def dataset_preparation_test():
    d = os.path.dirname(__file__)
    target = os.path.join(utils.get_project_root(),
                          'raw-datasets/unittests-tiny-raw.pickle')
    shutil.copyfile(os.path.join(d, 'data/unittests-tiny-raw.pickle'),
                    target)
    preprocess_dataset.create_preprocessed_dataset(
        target,
        os.path.join(utils.get_project_root(),
                     'preprocessed/small-baseline/data.pickle'),
        [preprocessing.ScaleAndShift()])


def get_parameters_test():
    # TODO: nose.proxy.UnicodeDecodeError: 'utf-8' codec can't decode byte 0x80
    #                                      in position 0: invalid start byte
    small = os.path.join(utils.get_project_root(),
                         "preprocessed/small-baseline")
    preprocess_dataset.get_parameters(small)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Test the 'serve' module of the hwrt package."""

import nose
from flask import Flask
from flask_bootstrap import Bootstrap
from nose.plugins.skip import SkipTest

# hwrt modules
import hwrt.serve as serve
import hwrt.utils as utils


# Tests
def execution_test():
    """Just test if the function is executable."""
    results = [{'symbolnr': 32,
                'semantics': '\\alpha',
                'probability': 0.67},
               {'symbolnr': 124,
                'semantics': '\\propto',
                'probability': 0.25}]
    serve.show_results(results)
    serve.get_json_result(results)
    serve.index()
    serve.get_parser()


@SkipTest
def fix_fix_writemath_answer_test():
    """Test if the function which brings the data into the format wanted by
       write-math.com works.
    """
    results = [{'symbolnr': 32,
                'semantics': '\\alpha',
                'probability': 0.67},
               {'symbolnr': 124,
                'semantics': '\\propto',
                'probability': 0.25}]
    serve.fix_writemath_answer(results)


def interactive_heartbeat_test():
    """Test the 'heartbeat' function of the '/interactive' route."""
    template_path = utils.get_template_folder()

    app = Flask(__name__, template_folder=template_path)
    Bootstrap(app)
    app.config.from_object(__name__)
    with app.test_request_context('/interactive?heartbeat=Peter'):
        a = serve.interactive()
        nose.tools.assert_equal(a, "Peter")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import nose

# hwrt modules
import hwrt.selfcheck as selfcheck


# Tests
def execution_test():
    # selfcheck.main()
    selfcheck.check_python_version()
    selfcheck.check_python_modules()


def which_test():
    return_value = selfcheck.which("somethingthatdoesntexist")
    nose.tools.assert_equal(return_value, None)
    return_value = selfcheck.which("/home/somethingthatdoesntexist")
    nose.tools.assert_equal(return_value, None)
    return_value = selfcheck.which("/bin/ls")
    nose.tools.assert_equal(return_value, "/bin/ls")



#!/usr/bin/env python

"""Script to import data into write-math.com"""

import logging
import sys

logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s',
                    level=logging.DEBUG,
                    stream=sys.stdout)

# hwrt modules
from hwrt.datasets import mfrdb
from hwrt import datasets


def main(directory):
    recordings = mfrdb.get_recordings(directory)
    logging.info("Got recordings for %i symbols.", len(recordings))
    recordings = sorted(recordings, key=lambda n: len(n[1]))
    for symbol, symbol_recs in recordings:
        logging.info("{0:>10}: {1} recordings".format(symbol,
                                                      len(symbol_recs)))
        for hw, info in symbol_recs:
            pass
            #datasets.insert_recording(hw, info)
    logging.info("Done importing dataset")


if __name__ == '__main__':
    main("/home/moose/Downloads/MfrDB_Symbols_v1.0")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os

# hwrt modules
import hwrt.create_model as create_model
import hwrt.utils as utils


# Tests
def execution_test():
    small = os.path.join(utils.get_project_root(), "models/small-baseline")
    create_model.main(small)


def parser_test():
    create_model.get_parser()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Helper functions for tests."""

import os

# hwrt modules
from hwrt.HandwrittenData import HandwrittenData


def get_all_symbols():
    current_folder = os.path.dirname(os.path.realpath(__file__))
    symbol_folder = os.path.join(current_folder, "symbols")
    symbols = [os.path.join(symbol_folder, f)
               for f in os.listdir(symbol_folder)
               if os.path.isfile(os.path.join(symbol_folder, f))]
    return symbols


def get_all_symbols_as_handwriting():
    handwritings = []
    for symbol_file in get_all_symbols():
        with open(symbol_file) as f:
            data = f.read()
        handwritings.append(HandwrittenData(data))
    return handwritings


def get_raw_datasets():
    raw_datasets = []
    for hwr in get_all_symbols_as_handwriting():
        hwr.formula_in_latex = 'TESTHELPER'
        hwr.formula_id = 42
        hwr.raw_data_id = 1337
        raw_datasets.append({'is_in_testset': 0,
                             'formula_id': hwr.formula_id,
                             'handwriting': hwr,
                             'formula_in_latex': hwr.formula_in_latex,
                             'id': hwr.raw_data_id})
    return raw_datasets


def get_symbol(raw_data_id):
    current_folder = os.path.dirname(os.path.realpath(__file__))
    file_path = os.path.join(current_folder, "symbols/%i.json" % raw_data_id)
    return file_path


def get_symbol_as_handwriting(raw_data_id):
    symbol_file = get_symbol(raw_data_id)
    with open(symbol_file) as f:
        data = f.read()
    a = HandwrittenData(data)
    return a


def compare_pointlists(a, b, epsilon=0.001):
    """Check if two stroke lists (a and b) are equal."""
    if len(a) != len(b):
        return False
    for stroke_a, stroke_b in zip(a, b):
        if len(stroke_a) != len(stroke_b):
            return False
        for point_a, point_b in zip(stroke_a, stroke_b):
            keys = ['x', 'y', 'time']
            for key in keys:
                if abs(point_a[key] - point_b[key]) > epsilon:
                    return False
    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import nose

# hwrt modules
import hwrt.geometry as geometry


# Tests
def object_creation_test():
    p1 = geometry.Point(0, 0)
    p2 = geometry.Point(1, 1)
    nose.tools.assert_equal(str(p1), "p(0.00, 0.00)")
    geometry.LineSegment(p1, p2)
    geometry.BoundingBox(p1, p2)
    geometry.PolygonalChain([{'x': 0, 'y': 0}, {'x': 10, 'y': 5},
                             {'x': 2, 'y': 3}])


def point_distance_test():
    a = geometry.Point(0, 0)
    b = geometry.Point(42, 0)
    nose.tools.assert_equal(a.dist_to(b), 42)
    nose.tools.assert_equal(b.dist_to(a), 42)


def line_segment_parallel_distance_test():
    """Test the distance of two parallel line segments."""
    # TODO: Other tests!
    ap1 = geometry.Point(0, 0)
    ap2 = geometry.Point(1, 0)
    la = geometry.LineSegment(ap1, ap2)

    bp1 = geometry.Point(0, 1)
    bp2 = geometry.Point(1, 1)
    lb = geometry.LineSegment(bp1, bp2)

    nose.tools.assert_equal(la.dist_to(lb), 1)

    bp1 = geometry.Point(0, 2)
    bp2 = geometry.Point(1, 2)
    lb = geometry.LineSegment(bp1, bp2)

    nose.tools.assert_equal(la.dist_to(lb), 2)

    bp1 = geometry.Point(1, 2)
    bp2 = geometry.Point(2, 2)
    lb = geometry.LineSegment(bp1, bp2)

    nose.tools.assert_equal(la.dist_to(lb), 2)


def bounding_box_test():
    p1 = geometry.Point(0, 0)
    p2 = geometry.Point(1, 1)
    bb = geometry.BoundingBox(p1, p2)
    nose.tools.assert_equal(bb.get_area(), 1)
    nose.tools.assert_equal(bb.get_width(), 1)
    nose.tools.assert_equal(bb.get_height(), 1)
    nose.tools.assert_equal(bb.get_largest_dimension(), 1)

    p1 = geometry.Point(0, 0)
    p2 = geometry.Point(2, 2)
    bb = geometry.BoundingBox(p1, p2)
    nose.tools.assert_equal(bb.get_area(), 4)
    nose.tools.assert_equal(bb.get_width(), 2)
    nose.tools.assert_equal(bb.get_height(), 2)
    nose.tools.assert_equal(bb.get_largest_dimension(), 2)

    p1 = geometry.Point(0, 0)
    p2 = geometry.Point(3, 2)
    bb = geometry.BoundingBox(p1, p2)
    nose.tools.assert_equal(bb.get_area(), 6)
    nose.tools.assert_equal(bb.get_width(), 3)
    nose.tools.assert_equal(bb.get_height(), 2)
    nose.tools.assert_equal(bb.get_largest_dimension(), 3)


def bb_intersection_test():
    ap1 = geometry.Point(0, 0)
    ap2 = geometry.Point(2, 2)
    bba = geometry.BoundingBox(ap1, ap2)

    # Test 1
    bp1 = geometry.Point(1, 1)
    bp2 = geometry.Point(2, 2)
    bbb = geometry.BoundingBox(bp1, bp2)
    nose.tools.assert_equal(geometry.do_bb_intersect(bba, bbb), True)

    # Test 2
    bp1 = geometry.Point(2, 2)
    bp2 = geometry.Point(3, 3)
    bbb = geometry.BoundingBox(bp1, bp2)
    nose.tools.assert_equal(geometry.do_bb_intersect(bba, bbb), True)

    # Test 3
    bp1 = geometry.Point(1, 1)
    bp2 = geometry.Point(3, 3)
    bbb = geometry.BoundingBox(bp1, bp2)
    nose.tools.assert_equal(geometry.do_bb_intersect(bba, bbb), True)

    # Test 3
    bp1 = geometry.Point(3, 3)
    bp2 = geometry.Point(4, 4)
    bbb = geometry.BoundingBox(bp1, bp2)
    nose.tools.assert_equal(geometry.do_bb_intersect(bba, bbb), False)


def segments_distance_test():
    ap1 = geometry.Point(0, 0)
    ap2 = geometry.Point(1, 0)
    la = geometry.LineSegment(ap1, ap2)

    # Test 1
    bp1 = geometry.Point(0, 1)
    bp2 = geometry.Point(1, 1)
    lb = geometry.LineSegment(bp1, bp2)
    nose.tools.assert_equal(geometry.segments_distance(la, lb), 1.0)

    # Test 2: Line segements cross
    bp1 = geometry.Point(0.5, 0.5)
    bp2 = geometry.Point(0.5, -0.5)
    lb = geometry.LineSegment(bp1, bp2)
    nose.tools.assert_equal(geometry.segments_distance(la, lb), 0)


def point_segment_distance_test():
    ap1 = geometry.Point(0, 0)
    ap2 = geometry.Point(1, 0)
    line = geometry.LineSegment(ap1, ap2)

    # Test 1
    point = geometry.Point(0, 0)
    nose.tools.assert_equal(geometry.point_segment_distance(point, line), 0.0)

    # Test 2
    point = geometry.Point(0, 1)
    nose.tools.assert_equal(geometry.point_segment_distance(point, line), 1.0)

    # Test 3: Line segement is just a point
    line = geometry.LineSegment(ap1, ap1)
    nose.tools.assert_equal(geometry.point_segment_distance(point, line), 1.0)

    # Test 3: Line segement is a straight vertical line
    ap1 = geometry.Point(0, 0)
    ap2 = geometry.Point(0, 1)
    line = geometry.LineSegment(ap1, ap2)
    point = geometry.Point(1, 0)
    nose.tools.assert_equal(geometry.point_segment_distance(point, line), 1.0)

    point = geometry.Point(0, 2)
    nose.tools.assert_equal(geometry.point_segment_distance(point, line), 1.0)

    point = geometry.Point(0, -1)
    nose.tools.assert_equal(geometry.point_segment_distance(point, line), 1.0)

    ap2 = geometry.Point(0, 0)
    ap1 = geometry.Point(0, 1)
    point = geometry.Point(0, -1)
    nose.tools.assert_equal(geometry.point_segment_distance(point, line), 1.0)

    ap1 = geometry.Point(0, 0)
    ap2 = geometry.Point(0, 1)
    line = geometry.LineSegment(ap2, ap1)
    point = geometry.Point(0, -1)
    nose.tools.assert_equal(geometry.point_segment_distance(point, line), 1.0)

    ap1 = geometry.Point(0, 0)
    ap2 = geometry.Point(0, 1)
    line = geometry.LineSegment(ap2, ap1)
    point = geometry.Point(0, 2)
    nose.tools.assert_equal(geometry.point_segment_distance(point, line), 1.0)

    ap1 = geometry.Point(0, 0)
    ap2 = geometry.Point(1, 0)
    line = geometry.LineSegment(ap2, ap1)
    point = geometry.Point(2, 0)
    nose.tools.assert_equal(geometry.point_segment_distance(point, line), 1.0)

    ap1 = geometry.Point(0, 0)
    ap2 = geometry.Point(1, 0)
    line = geometry.LineSegment(ap1, ap2)
    point = geometry.Point(2, 0)
    nose.tools.assert_equal(geometry.point_segment_distance(point, line), 1.0)

    ap1 = geometry.Point(0, 0)
    ap2 = geometry.Point(1, 0)
    line = geometry.LineSegment(ap1, ap2)
    point = geometry.Point(-1, 0)
    nose.tools.assert_equal(geometry.point_segment_distance(point, line), 1.0)

    ap1 = geometry.Point(0, 0)
    ap2 = geometry.Point(1, 0)
    line = geometry.LineSegment(ap2, ap1)
    point = geometry.Point(-1, 0)
    nose.tools.assert_equal(geometry.point_segment_distance(point, line), 1.0)

    # Test 3: Line segement is a straight vertical line and point is on it
    ap1 = geometry.Point(0, 0)
    ap2 = geometry.Point(0, 1)
    line = geometry.LineSegment(ap1, ap2)
    point = geometry.Point(0, 0.1)
    nose.tools.assert_equal(geometry.point_segment_distance(point, line), 0)

    # Test 4: Vertical line tests
    ap1 = geometry.Point(0, 0)
    ap2 = geometry.Point(0, 1)
    line = geometry.LineSegment(ap1, ap2)
    point = geometry.Point(0.5, 0.5)
    nose.tools.assert_equal(geometry.point_segment_distance(point, line), 0.5)

    ap1 = geometry.Point(0, 0)
    ap2 = geometry.Point(0, 1)
    line = geometry.LineSegment(ap1, ap2)
    point = geometry.Point(-0.5, 0.5)
    nose.tools.assert_equal(geometry.point_segment_distance(point, line), 0.5)

    # Test 5: Other
    ap1 = geometry.Point(0, 0)
    ap2 = geometry.Point(1, 1)
    line = geometry.LineSegment(ap1, ap2)
    point = geometry.Point(1, 0)
    nose.tools.assert_equal(geometry.point_segment_distance(point, line),
                            (2**0.5)/2.0)

    # Test 6: Continued line
    ap1 = geometry.Point(0, 0)
    ap2 = geometry.Point(1, 1)
    line = geometry.LineSegment(ap1, ap2)
    point = geometry.Point(2, 2)
    nose.tools.assert_equal(geometry.point_segment_distance(point, line),
                            2**0.5)

    ap1 = geometry.Point(0, 0)
    ap2 = geometry.Point(1, 1)
    line = geometry.LineSegment(ap1, ap2)
    point = geometry.Point(-1, -1)
    nose.tools.assert_equal(geometry.point_segment_distance(point, line),
                            2**0.5)


def segments_intersection_test():
    l1 = geometry.LineSegment(geometry.Point(0, 0), geometry.Point(2, 2))
    l2 = geometry.LineSegment(geometry.Point(1, 1), geometry.Point(3, 3))
    nose.tools.assert_equal(geometry.get_segments_intersections(l1, l2),
                            [geometry.Point(1, 1)])

    l1 = geometry.LineSegment(geometry.Point(0, 0), geometry.Point(2, 2))
    l2 = geometry.LineSegment(geometry.Point(2.1, 2.1), geometry.Point(3, 3))
    nose.tools.assert_equal(geometry.get_segments_intersections(l1, l2), [])

    # 2 vertical line segments
    l1 = geometry.LineSegment(geometry.Point(42, 2), geometry.Point(42, 0))
    l2 = geometry.LineSegment(geometry.Point(42, 1), geometry.Point(42, -1))
    nose.tools.assert_equal(geometry.get_segments_intersections(l1, l2),
                            [geometry.Point(42, 0)])

    l1 = geometry.LineSegment(geometry.Point(-1, 0), geometry.Point(1, 0))
    l2 = geometry.LineSegment(geometry.Point(0, -1), geometry.Point(0, 1))
    nose.tools.assert_equal(geometry.get_segments_intersections(l1, l2),
                            [geometry.Point(0, 0)])


def segment_intersection_test():
    l1 = geometry.LineSegment(geometry.Point(289, 69), geometry.Point(290, 69))
    l2 = geometry.LineSegment(geometry.Point(291, 69), geometry.Point(292, 69))
    nose.tools.assert_equal(geometry.get_segments_intersections(l1, l2), [])

    l1 = geometry.LineSegment(geometry.Point(0, 0), geometry.Point(0, 10))
    l2 = geometry.LineSegment(geometry.Point(5, 5), geometry.Point(2, 0))
    nose.tools.assert_equal(geometry.get_segments_intersections(l1, l2), [])


def stroke_selfintersection_test():
    a = [{'y': 69, 'x': 289},
         {'y': 69, 'x': 290},
         {'y': 69, 'x': 291},
         {'y': 69, 'x': 292},
         {'y': 69, 'x': 293},
         {'y': 69, 'x': 295},
         {'y': 70, 'x': 297},
         {'y': 70, 'x': 299},
         {'y': 71, 'x': 302},
         {'y': 72, 'x': 303}]
    a = geometry.PolygonalChain(a)
    nose.tools.assert_equal(a.count_selfintersections(), 0)



#!/usr/bin/env python

"""Testing script for write-math symbol classifier."""

import logging
import sys
import json
import numpy

logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s',
                    level=logging.DEBUG,
                    stream=sys.stdout)

# hwrt modules
from hwrt.classify import classify_segmented_recording as evaluate
from hwrt.datasets import mfrdb


def _less_than(l, n):
    """Get number of symbols in list `l` which have a value less than `n`.

    Parameters
    ----------
    l : list
        List of numbers
    n : int

    Returns
    -------
    float (with int values)
    """
    return float(len([1 for el in l if el < n]))


def main(directory):
    recordings = mfrdb.get_recordings(directory)
    print(len(recordings))

    score_place = []
    symbols_showed = []
    for latex, symbol_recording in recordings:
        score_place_symbol = []
        for recording in symbol_recording:
            results = evaluate(json.dumps(recording.get_sorted_pointlist()),
                               result_format='LaTeX')
            for i, result in enumerate(results):
                if result['semantics'] == recording.formula_in_latex:
                    # if i > 5:
                    #     logging.info("  Found '%s' place %i with probability %0.4f.",
                    #                  recording.formula_in_latex,
                    #                  i,
                    #                  result['probability'])
                    #     if recording.formula_in_latex not in symbols_showed:
                    #         #recording.show()
                    #         symbols_showed.append(recording.formula_in_latex)
                    score_place.append(i)
                    score_place_symbol.append(i)
                    break
            else:
                if recording.formula_in_latex not in symbols_showed:
                    logging.info("#"*80)
                    logging.info(recording.formula_in_latex)
                    symbols_showed.append(recording.formula_in_latex)
        logging.info(("{0:>10}: TOP-1: {1:0.2f} | TOP-3: {2:0.2f} | "
                      "TOP-10: {3:0.2f} | TOP-50: {4:0.2f} | {5}").format(
                     latex,
                     _less_than(score_place_symbol, 1)/len(symbol_recording),
                     _less_than(score_place_symbol, 3)/len(symbol_recording),
                     _less_than(score_place_symbol, 10)/len(symbol_recording),
                     _less_than(score_place_symbol, 50)/len(symbol_recording),
                     len(symbol_recording)))

    total_recordings = len([1 for l, symb in recordings for el in symb])
    logging.info("mean place of correct classification: %0.2f",
                 numpy.mean(score_place))
    logging.info("median place of correct classification: %0.2f",
                 numpy.median(score_place))
    for i in [1, 3, 10, 20, 50]:
        logging.info("TOP-%i: %0.2f correct",
                     i,
                     _less_than(score_place, i)/total_recordings)
    #logging.info("Out of order: %i", out_of_order_count)
    logging.info("Total: %i", total_recordings)


if __name__ == '__main__':
    recordings = main("/home/moose/Downloads/MfrDB_Symbols_v1.0")



#!/usr/bin/env python

"""Normalize mathematical formulae written with LaTeX."""

import string


def chunk_math(text):
    """
    Parameters
    ----------
    text : string
        A mathematical context

    Returns
    -------
    list :
        A list of single LaTeX entities

    Examples
    --------
    >>> chunk_math('\sum_i^n i^2')
    ['\\\\sum', '_', 'i', '^', 'n', ' ', 'i', '^', '2']

    >>> chunk_math('\sum_{i}^n i^2')
    ['\\\\sum', '_', '{', 'i', '}', '^', 'n', ' ', 'i', '^', '2']

    >>> chunk_math((r'\\Delta F_0 &= \\sqrt{\\sum_{i=1}^n\\left('
    ...             r'\\frac{\delta F_0}{\delta x_i}'
    ...             r'\Delta x_i\\right)^2}\\[0.2cm]'
    ...             r'\Delta F_0 &= \sqrt{6.044 \cdot  10^{-6}\\text{m}^2}'))
    ['\\\\Delta', ' ', 'F', '_', '0', ' ', '&', '=', ' ', '\\\\sqrt', '{', '\\\\sum', '_', '{', 'i', '=', '1', '}', '^', 'n', '\\\\left', '(', '\\\\frac', '{', '\\\\delta', ' ', 'F', '_', '0', '}', '{', '\\\\delta', ' ', 'x', '_', 'i', '}', '\\\\Delta', ' ', 'x', '_', 'i', '\\\\right', ')', '^', '2', '}', '\\\\', '[', '0', '.', '2', 'c', 'm', ']', '\\\\Delta', ' ', 'F', '_', '0', ' ', '&', '=', ' ', '\\\\sqrt', '{', '6', '.', '0', '4', '4', ' ', '\\\\cdot', ' ', '1', '0', '^', '{', '-', '6', '}', '\\\\text', '{', 'm', '}', '^', '2', '}']

    >>> chunk_math(r'\\left\\{a\\right\\}')
    ['\\\\left', '\\\\{', 'a', '\\\\right', '\\\\}']

    >>> chunk_math(r'\\sqrt{b^2-4ac}')
    ['\\\\sqrt', '{', 'b', '^', '2', '-', '4', 'a', 'c', '}']

    >>> chunk_math('y^{2}')
    ['y', '^', '{', '2', '}']

    >>> chunk_math(r'2+3\\\\6 5 4')
    ['2', '+', '3', '\\\\\\\\', '6', ' ', '5', ' ', '4']
    """

    # Fail when '{' and '}' don't match - be aware of escaped symbols!
    opened_braces = 0
    last_char = ''
    for char in text:
        if char == '{' and last_char != '\\':
            opened_braces += 1
        if char == '}' and last_char != '\\':
            opened_braces -= 1
            if opened_braces < 0:
                raise ValueError("Braces don't match: %s" % text)
        last_char = char
    if opened_braces != 0:
        raise ValueError("%i braces are still open" % opened_braces)

    # Parse
    single_symbol = ['_', '^', '&', '{', '}']
    breaking_chars = ['\\', ' '] + single_symbol

    chunks = []
    current_chunk = ''

    for char in text:
        if current_chunk == '':
            current_chunk = char
            continue
        if char == '\\':
            if current_chunk == '\\':
                current_chunk += char
                chunks.append(current_chunk)
                current_chunk = ''
            else:
                chunks.append(current_chunk)
                current_chunk = char
        elif current_chunk == '\\' and char in breaking_chars:  # escaped
            current_chunk += char
            chunks.append(current_chunk)
            current_chunk = ''
        elif char in breaking_chars:
            chunks.append(current_chunk)
            current_chunk = char
        elif char in string.letters+string.digits and current_chunk[0] == '\\':
            current_chunk += char
        else:
            chunks.append(current_chunk)
            current_chunk = char

    # Add the last chunk
    if current_chunk != '':
        chunks.append(current_chunk)
    filtered = []
    for chunk in chunks:
        if len(filtered) > 0 and filtered[-1] == ' ' and chunk == ' ':
            continue
        filtered.append(chunk)
    return filtered


def chunks_to_string(chunks):
    """
    Parameters
    ----------
    chunks : list of strings
        A list of single entities in order

    Returns
    -------
    string :
        A LaTeX-parsable string

    Examples
    --------
    >>> chunks_to_string(['\\\\sum', '_', 'i', '^', 'n', 'i', '^', '2'])
    '\\\\sum_{i}^{n}i^{2}'
    >>> chunks_to_string(['\\\\sum', '_', '{', 'i', '}', '^', 'n', 'i', '^',
    ...                   '2'])
    '\\\\sum_{i}^{n}i^{2}'
    """
    string = ''
    began_context = False
    context_depth = 0
    context_triggers = ['_', '^']
    for chunk in chunks:
        if began_context and chunk != '{':
            string += '{' + chunk + '}'
            began_context = False
        elif began_context and chunk == '{':
            began_context = False
            string += chunk
        else:
            if chunk in context_triggers:
                began_context = True
                context_depth += 1
            string += chunk
    return string


def normalize(latex):
    """Normalize a math-mode LaTeX string.

    Parameters
    ----------
    latex : string

    Returns
    -------
    string : normalized LaTeX code

    Examples
    --------
    >>> normalize('\sum_i n^i')
    '\\\\sum_{i} n^{i}'
    >>> normalize('Carsten Wilhelm')
    'Carsten Wilhelm'
    >>> normalize(r'\\Delta F')
    '\\\\Delta F'
    >>> normalize(r'\\sqrt{b^2-4ac}')
    '\\\\sqrt{b^{2}-4ac}'
    >>> normalize(r'y^{2}')
    'y^{2}'
    """
    return chunks_to_string(chunk_math(latex))


if __name__ == '__main__':
    import doctest
    doctest.testmod()



#!/usr/bin/env python

"""Evaluate a folder of InkML files for a CROHME competition."""

import glob

import json
import logging
import sys

logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s',
                    level=logging.DEBUG,
                    stream=sys.stdout)

# HWRT modules
from hwrt.datasets import inkml
from hwrt.classify import classify_segmented_recording as evaluate


def evaluate_dir(sample_dir):
    """Evaluate all recordings in `sample_dir`.

    Parameters
    ----------
    sample_dir : string
        The path to a directory with *.inkml files.

    Returns
    -------
    list of dictionaries
        Each dictionary contains the keys 'filename' and 'results', where
        'results' itself is a list of dictionaries. Each of the results has
        the keys 'latex' and 'probability'
    """
    results = []
    if sample_dir[-1] == "/":
        sample_dir = sample_dir[:-1]
    for filename in glob.glob("%s/*.inkml" % sample_dir):
        results.append(evaluate_inkml(filename))
    return results


def evaluate_inkml(inkml_file_path):
    """Evaluate an InkML file.

    Parameters
    ----------
    inkml_file_path : string
        path to an InkML file

    Returns
    -------
    dictionary
        The dictionary contains the keys 'filename' and 'results', where
        'results' itself is a list of dictionaries. Each of the results has the
        keys 'semantics' (which contains the latex command) and 'probability'
    """
    logging.info("Start evaluating '%s'...", inkml_file_path)
    ret = {'filename': inkml_file_path}
    recording = inkml.read(inkml_file_path)
    results = evaluate(json.dumps(recording.get_sorted_pointlist()),
                       result_format='LaTeX')
    ret['results'] = results
    return ret


def generate_output_csv(evaluation_results, filename='results.csv'):
    """Generate the evaluation results in the format

    Parameters
    ----------
    evaluation_results : list of dictionaries
        Each dictionary contains the keys 'filename' and 'results', where
        'results' itself is a list of dictionaries. Each of the results has
        the keys 'latex' and 'probability'

    Examples
    --------
    MfrDB3907_85801, a, b, c, d, e, f, g, h, i, j
    scores, 1, 0.9, 0.8, 0.7, 0.6, 0.5, 0.4, 0.3, 0.2, 0.1
    MfrDB3907_85802, 1, |, l, COMMA, junk, x, X, \times
    scores, 10, 8.001, 2, 0.5, 0.1, 0,-0.5, -1, -100
    """
    with open(filename, 'w') as f:
        for result in evaluation_results:
            for i, entry in enumerate(result['results']):
                if entry['semantics'] == ',':
                    result['results']['semantics'] = 'COMMA'
            f.write("%s, " % result['filename'])
            f.write(", ".join([entry['semantics'] for entry in result['results']]))
            f.write("\n")
            f.write("%s, " % "scores")
            f.write(", ".join([str(entry['probability']) for entry in result['results']]))
            f.write("\n")


def get_parser():
    from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
    parser = ArgumentParser(description=__doc__,
                            formatter_class=ArgumentDefaultsHelpFormatter)
    parser.add_argument("-d", "--dir",
                        dest="sample_dir",
                        help="directory with data to evaluate",
                        required=True,
                        metavar="DIRECTORY")
    return parser


def main(sample_dir):
    evaluation_results = evaluate_dir(sample_dir)
    generate_output_csv(evaluation_results)


if __name__ == "__main__":
    args = get_parser().parse_args()
    main(args.sample_dir)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import tests.testhelper as testhelper

# hwrt modules
import hwrt.analyze_data as analyze_data
import hwrt.features as features


# Tests
def execution_test():
    analyze_data.filter_label("\\dag", replace_by_similar=True)
    analyze_data.filter_label("\\diameter", replace_by_similar=True)
    analyze_data.filter_label("\\degree", replace_by_similar=True)
    analyze_data.filter_label("\\alpha", replace_by_similar=True)

    raw_datasets = testhelper.get_raw_datasets()
    feature = features.AspectRatio()
    filename = "aspect_ratio.csv"
    analyze_data.analyze_feature(raw_datasets, feature, filename)


def parser_test():
    analyze_data.get_parser()



#!/usr/bin/env python

"""Read and parse data from the MathBrush project."""

import os
from natsort import natsorted
import glob
import json
import re

import logging
import sys

logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s',
                    level=logging.DEBUG,
                    stream=sys.stdout)

# hwrt modules
from hwrt import HandwrittenData
from hwrt import datasets

missing_stroke_segmentation = []
double_segmentation = []


def remove_matching_braces(latex):
    if latex.startswith('{') and latex.endswith('}'):
        opened = 1
        matches = True
        for char in latex[1:-1]:
            if char == '{':
                opened += 1
            elif char == '}':
                opened -= 1
            if opened == 0:
                matches = False
        if matches:
            latex = latex[1:-1]
    return latex


def get_latex(ink_filename):
    """Get the LaTeX string from a file by the *.ink filename."""
    tex_file = os.path.splitext(ink_filename)[0] + ".tex"
    with open(tex_file) as f:
        tex_content = f.read().strip()
    pattern = re.compile(r"\\begin\{displaymath\}(.*?)\\end\{displaymath\}",
                         re.DOTALL)
    matches = pattern.findall(tex_content)

    if len(matches) == 0:
        pattern = re.compile(r"$$(.*?)$$",
                             re.DOTALL)
        matches = pattern.findall(tex_content)

    if len(matches) != 1:
        raise Exception("%s: Found not one match, but %i: %s" %
                        (ink_filename, len(matches), matches))
    formula_in_latex = matches[0].strip()
    formula_in_latex = remove_matching_braces(formula_in_latex)

    # repl = []
    # for letter in string.letters:
    #     repl.append(('\mbox{%s}' % letter, letter))
    # for search, replace in repl:
    #     formula_in_latex = formula_in_latex.replace(search, replace)
    return formula_in_latex


def get_segmentation(recording, annotations, internal_id=None):
    global missing_stroke_segmentation, double_segmentation
    segmentation = []
    needed = list(range(len(recording)))
    annotations = filter(lambda n: n.startswith('SYMBOL '), annotations)
    for line in annotations:
        tmp = line.split("<")[1]
        tmp = tmp.split(">")[0]
        strokes = [int(stroke) for stroke in tmp.split(",")
                   if int(stroke) < len(recording)]
        for el in strokes:
            if el not in needed:
                double_segmentation.append(internal_id)
                # hw = HandwrittenData.HandwrittenData(json.dumps(recording))
                # hw.show()
                strokes.remove(el)
                logging.debug("invalid segmentation by annotation: %s",
                              annotations)
            else:
                needed.remove(el)
        segmentation.append(strokes)
    if len(needed) > 0:
        # hw = HandwrittenData.HandwrittenData(json.dumps(recording))
        # hw.show()
        # print(annotations)
        # print(needed)
        missing_stroke_segmentation.append(internal_id)
        logging.debug(("invalid segmentation by annotation: %s "
                       "(needed: %s)"), annotations, needed)
        segmentation.append(needed)
    return segmentation


def parse_scg_ink_file(filename):
    """Parse a SCG INK file.

    Parameters
    ----------
    filename : string
        The path to a SCG INK file.

    Returns
    -------
    HandwrittenData
        The recording as a HandwrittenData object.
    """
    stroke_count = 0
    stroke_point_count = -1
    recording = []
    current_stroke = []
    time = 0
    got_annotations = False
    annotations = []

    formula_in_latex = get_latex(filename)

    with open(filename) as f:
        contents = f.read().strip()
    lines = contents.split("\n")
    for i, line in enumerate(lines):
        line = line.strip()
        if i == 0 and line != 'SCG_INK':
            raise ValueError(("%s: SCG Ink files have to start with 'SCG_INK'."
                              " The file started with %s.") %
                             (filename, line))
        elif i == 1:
            try:
                stroke_count = int(line)
            except ValueError:
                raise ValueError(("%s: Second line has to be the number of "
                                  "strokeswhich has to be an integer, but "
                                  "was '%s'") % (filename, line))
            if stroke_count <= 0:
                # os.remove(filename)
                # return []
                raise ValueError(("%s: Stroke count was %i, but should be "
                                  "> 0.") % (filename, stroke_count))
        elif i == 2:
            try:
                stroke_point_count = int(line)
            except ValueError:
                raise ValueError("%s: Third line has to be the number of "
                                 "points which has to be an integer, but was "
                                 "'%s'" % (filename, line))
            if stroke_point_count <= 0:
                raise ValueError(("%s: Stroke point count was %i, but should "
                                  "be > 0.") % (filename, stroke_count))
        elif i > 2:
            if stroke_point_count > 0:
                x, y = [int(el) for el in line.strip().split(" ")]
                current_stroke.append({'x': x, 'y': y, 'time': time})
                time += 20
                stroke_point_count -= 1
            elif line == 'ANNOTATIONS' or got_annotations:
                got_annotations = True
                annotations.append(line)
            elif stroke_count > 0:
                try:
                    stroke_point_count = int(line)
                except ValueError:
                    raise ValueError(("%s: Line %i has to be the number of "
                                      "points which has to be an integer, "
                                      " but was '%s'") % (filename, i+1, line))
                if stroke_point_count <= 0:
                    raise ValueError(("%s: Stroke point count was %i, but "
                                      "should be > 0.") %
                                     (filename, stroke_count))
            if stroke_point_count == 0 and len(current_stroke) > 0:
                time += 200
                recording.append(current_stroke)
                stroke_count -= 1
                current_stroke = []
    hw = HandwrittenData.HandwrittenData(json.dumps(recording),
                                         formula_in_latex=formula_in_latex,
                                         formula_id=datasets.formula_to_dbid(formula_in_latex))
    hw.internal_id = "/".join(filename.split("/")[-2:])
    hw.segmentation = get_segmentation(recording, annotations, hw.internal_id)
    hw.description = "\n".join(annotations)
    hw.username = "MathBrush::%s" % os.path.basename(os.path.dirname(filename))
    copyright_str = ("This dataset was contributed by MathBrush. You can "
                     "download their complete dataset by contacting them. See "
                     "[www.scg.uwaterloo.ca/mathbrush/]"
                     "(https://www.scg.uwaterloo.ca/mathbrush/corpus.shtml)")
    hw.user_id = datasets.getuserid(hw.username, copyright_str)
    #logging.info("'%s' by '%s'", hw.formula_in_latex, hw.username)
    #hw.show()
    return hw


def read_folder(folder):
    """Read all files of `folder` and return a list of HandwrittenData
    objects.
    """
    recordings = []
    for filename in glob.glob(os.path.join(folder, '*.ink')):
        #print(filename)
        recording = parse_scg_ink_file(filename)
        recordings.append(recording)
    return recordings


def main(directory):
    user_dirs = natsorted(list(next(os.walk(directory))[1]))
    recordings = []
    for user_dir in user_dirs:
        user_dir = os.path.join(directory, user_dir)
        logging.info("Start getting data from %s...", user_dir)
        recordings += read_folder(user_dir)
        # for el in recordings:
        #     print(el)
    logging.info("Got %i recordings.", len(recordings))
    logging.info("Double segmented strokes: %i (%0.2f%%)",
                 len(double_segmentation),
                 float(len(double_segmentation))/len(recordings))
    logging.info("Missing segmented strokes: %i (%0.2f%%)",
                 len(missing_stroke_segmentation),
                 float(len(missing_stroke_segmentation))/len(recordings))
    for recording in recordings:
        datasets.insert_recording(recording)

if __name__ == '__main__':
    main("/home/moose/Downloads/mathbrush/mathdata")



#!/usr/bin/env python

import logging
import sys
import datetime

logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s',
                    level=logging.DEBUG,
                    stream=sys.stdout)

import os

from hwrt.datasets import inkml
from hwrt.datasets import getuserid, insert_recording


def main(directory):
    recordings = inkml.read_folder(directory)
    for hw in recordings:
        hw.creation_date = datetime.datetime.fromtimestamp(hw.get_sorted_pointlist()[0][0]['time']/1000.0)
        hw.internal_id = hw.filepath
        # username
        username = os.path.basename(hw.filepath)
        username = username.split('_')[1]
        username = username.split('.')[0]
        hw.username = "expressmatch::%s" % username
        print(hw.username)
        # insert user
        copyright_str = ("This dataset was contributed by "
                         "[express-match](https://code.google.com/p/express-match/)."
                         "It is described in the paper "
                         "'ExpressMatch: A System for Creating Ground-Truthed "
                         "Datasets of Online Mathematical Expressions'.")
        hw.user_id = getuserid(hw.username, copyright_str)
        # insert recording
        insert_recording(hw)
        print(hw.symbol_stream)
        print(hw.segmentation)
        #hw.show()


if __name__ == '__main__':
    main("/home/moose/Downloads/expressmatch-time-datasetV0.2")



#!/usr/bin/env python

"""Classify recordings."""

import logging
import pkg_resources
import os

# hwrt modules
from hwrt import utils

single_symbol_classifier = None


class single_classificer(object):
    """Classifier for single (and hence perfectly segmented) symbols."""

    def __init__(self):
        logging.info("Start reading model...")
        model_path = pkg_resources.resource_filename('hwrt', 'misc/')
        model_file = os.path.join(model_path, "model.tar")
        logging.info("Model: %s", model_file)
        (preprocessing_queue, feature_list, model,
         output_semantics) = utils.load_model(model_file)
        self.preprocessing_queue = preprocessing_queue
        self.feature_list = feature_list
        self.model = model
        self.output_semantics_orig = output_semantics
        self.output_semantics = self.output_semantics_orig
        if 'HWRT_ENV' in os.environ and os.environ['HWRT_ENV'] == 'dev':
            new_semantics = []
            for el in output_semantics:
                new_semantics.append(el.split(";")[1])
            self.output_semantics = new_semantics

    def predict(self, recording, result_format=None):
        """Predict the class of the given recording.

        Parameters
        ----------
        recording : string
            Recording of a single handwritten dataset in JSON format.
        result_format : string, optional
            If it is 'LaTeX', then only the latex code will be returned

        Returns
        -------
        list
        """
        evaluate = utils.evaluate_model_single_recording_preloaded
        results = evaluate(self.preprocessing_queue,
                           self.feature_list,
                           self.model,
                           self.output_semantics,
                           recording)
        if result_format == 'LaTeX':
            for i in range(len(results)):
                results[i]['semantics'] = results[i]['semantics'].split(";")[1]
        return results


def classify_segmented_recording(recording, result_format=None):
    """Use this function if you are sure you have a single symbol.

    Parameters
    ----------
    recording : string
        The recording in JSON format

    Returns
    -------
    list of dictionaries
        Each dictionary contains the keys 'symbol' and 'probability'. The list
        is sorted descending by probability.
    """
    global single_symbol_classifier
    if single_symbol_classifier is None:
        single_symbol_classifier = single_classificer()
    return single_symbol_classifier.predict(recording, result_format)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import nose
import tests.testhelper as testhelper

# hwrt modules
from hwrt.HandwrittenData import HandwrittenData
import hwrt.preprocessing as preprocessing


# Tests
def preprocessing_detection_test():
    preprocessing_queue = [{'ScaleAndShift': None},
                           {'StrokeConnect': None},
                           {'DouglasPeucker': [{'epsilon': 0.2}]},
                           {'SpaceEvenly': [{'number': 100}]}]
    correct = [preprocessing.ScaleAndShift(),
               preprocessing.StrokeConnect(),
               preprocessing.DouglasPeucker(epsilon=0.2),
               preprocessing.SpaceEvenly(number=100)]
    feature_list = preprocessing.get_preprocessing_queue(preprocessing_queue)
    # TODO: Not only compare lengths of lists but actual contents.
    nose.tools.assert_equal(len(feature_list), len(correct))


def simple_execution_test():
    algorithms = [preprocessing.RemoveDuplicateTime(),
                  preprocessing.RemoveDots(),
                  preprocessing.SpaceEvenly(),
                  preprocessing.SpaceEvenlyPerStroke(),
                  preprocessing.DouglasPeucker(),
                  preprocessing.StrokeConnect(),
                  preprocessing.DotReduction(),
                  preprocessing.WildPointFilter(),
                  preprocessing.WeightedAverageSmoothing()]
    for algorithm in algorithms:
        a = testhelper.get_symbol_as_handwriting(292934)
        algorithm(a)


def euclidean_distance_test():
    p1 = {'x': 12, 'y': 15}
    p2 = {'x':  2, 'y': 50}
    dist = preprocessing.euclidean_distance(p1, p2)
    nose.tools.assert_equal(round(dist, 1), 36.4)


def ScaleAndShift_test_all():
    preprocessing_queue = [preprocessing.ScaleAndShift()]
    for a in testhelper.get_all_symbols_as_handwriting():
        a.preprocessing(preprocessing_queue)
        s = a.get_pointlist()
        assert len(s) > 0


def ScaleAndShift_test_simple_1():
    preprocessing_queue = [preprocessing.ScaleAndShift()]
    s = '[[{"x":0, "y":0, "time": 0}]]'
    a = HandwrittenData(s)
    a.preprocessing(preprocessing_queue)
    s = a.get_pointlist()
    expectation = [[{"x": 0, "y": 0, "time": 0}]]
    assert s == expectation, "Got: %s; expected %s" % (s, expectation)


def ScaleAndShift_test_simple_2():
    preprocessing_queue = [preprocessing.ScaleAndShift()]
    s = '[[{"x":10, "y":0, "time": 0}]]'
    a = HandwrittenData(s)
    a.preprocessing(preprocessing_queue)
    s = a.get_pointlist()
    expectation = [[{"x": 0, "y": 0, "time": 0}]]
    assert s == expectation, "Got: %s; expected %s" % (s, expectation)


def ScaleAndShift_test_simple_3():
    preprocessing_queue = [preprocessing.ScaleAndShift()]
    s = '[[{"x":0, "y":10, "time": 0}]]'
    a = HandwrittenData(s)
    a.preprocessing(preprocessing_queue)
    s = a.get_pointlist()
    expectation = [[{"x": 0, "y": 0, "time": 0}]]
    assert s == expectation, "Got: %s; expected %s" % (s, expectation)


def ScaleAndShift_test_simple_4():
    preprocessing_queue = [preprocessing.ScaleAndShift()]
    s = '[[{"x":0, "y":0, "time": 10}]]'
    a = HandwrittenData(s)
    a.preprocessing(preprocessing_queue)
    s = a.get_pointlist()
    expectation = [[{"x": 0, "y": 0, "time": 0}]]
    assert s == expectation, "Got: %s; expected %s" % (s, expectation)


def ScaleAndShift_test_simple_5():
    preprocessing_queue = [preprocessing.ScaleAndShift()]
    s = '[[{"x":42, "y":12, "time": 10}]]'
    a = HandwrittenData(s)
    a.preprocessing(preprocessing_queue)
    s = a.get_pointlist()
    expectation = [[{"x": 0, "y": 0, "time": 0}]]
    assert s == expectation, "Got: %s; expected %s" % (s, expectation)


def ScaleAndShift_test_a():
    preprocessing_queue = [preprocessing.ScaleAndShift()]
    s = ('[[{"x":232,"y":423,"time":1407885913983},'
         '{"x":267,"y":262,"time":1407885914315},'
         '{"x":325,"y":416,"time":1407885914650}],'
         '[{"x":252,"y":355,"time":1407885915675},'
         '{"x":305,"y":351,"time":1407885916361}]]')
    a = HandwrittenData(s)
    a.preprocessing(preprocessing_queue)
    s = a.get_pointlist()
    expectation = [[{u'y': 1.0, u'x': 0.0, u'time': 0},
                    {u'y': 0.0, u'x': 0.2174, u'time': 332},
                    {u'y': 0.9565, u'x': 0.5776, u'time': 667}],
                   [{u'y': 0.5776, u'x': 0.1242, u'time': 1692},
                    {u'y': 0.5528, u'x': 0.4534, u'time': 2378}]]
    assert testhelper.compare_pointlists(s, expectation), \
        "Got: %s; expected %s" % (s, expectation)


def ScaleAndShift_test_a_center():
    preprocessing_queue = [preprocessing.ScaleAndShift(center=True)]
    s = ('[[{"y": 1.0, "x": -0.3655913978494625, "time": 0}, '
         '{"y": 0.0, "x": -0.1482000935016364, "time": 332}, '
         '{"y": 0.9565, "x": 0.21204835370333253, "time": 667}], '
         '[{"y": 0.5776, "x": -0.24136779536499045, "time": 1692}, '
         '{"y": 0.5528, "x": 0.08782475121886046, "time": 2378}]]')
    a = HandwrittenData(s)
    a.preprocessing(preprocessing_queue)
    s = a.get_pointlist()
    expectation = [[{u'y': 1.0, u'x': -0.2888198757763975, u'time': 0},
                    {u'y': 0.0, u'x': -0.07142857142857142, u'time': 332},
                    {u'y': 0.9565, u'x': 0.2888198757763975, u'time': 667}],
                   [{u'y': 0.5776, u'x': -0.16459627329192547, u'time': 1692},
                    {u'y': 0.5528, u'x': 0.16459627329192544, u'time': 2378}]]
    assert testhelper.compare_pointlists(s, expectation), \
        "Got: %s; expected %s" % (s, expectation)


def space_evenly_per_stroke_test_all():
    preprocessing_queue = [preprocessing.SpaceEvenlyPerStroke(number=100,
                                                              kind='cubic')]
    for a in testhelper.get_all_symbols_as_handwriting():
        a.preprocessing(preprocessing_queue)
        s = a.get_pointlist()
        print(s)
        assert len(s) > 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import nose

import tests.testhelper as testhelper

# hwrt modules
import hwrt.data_multiplication as data_multiplication


# Tests
def data_multiplication_detection_test():
    l = [{'Multiply': None},
         {'Multiply': [{'nr': 1}]},
         {'Rotate':
          [{'minimum': -3},
           {'maximum': +3},
           {'num': 3}]
          }
         ]
    correct = [data_multiplication.Multiply(nr=1),
               data_multiplication.Multiply(nr=1),
               data_multiplication.Rotate(minimum=-3,
                                          maximum=3,
                                          num=3)]
    mult_queue = data_multiplication.get_data_multiplication_queue(l)
    # TODO: Not only compare lengths of lists but actual contents.
    nose.tools.assert_equal(len(mult_queue), len(correct))


def rotate_test():
    recording = testhelper.get_symbol_as_handwriting(292934)
    rotation = data_multiplication.Rotate(minimum=-3, maximum=3, num=3)
    new_recordings = rotation(recording)
    # TODO: Not only compare lengths of lists but actual contents.
    nose.tools.assert_equal(len(new_recordings), 3)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import nose
import tests.testhelper as th

# hwrt modules
from hwrt.HandwrittenData import HandwrittenData
import hwrt.preprocessing as preprocessing
import hwrt.features as features
import hwrt.create_pfiles as create_pfiles
import hwrt.data_multiplication as data_multiplication
import hwrt.utils as utils


# Tests
def training_set_multiplication_test():
    sample = th.get_symbol_as_handwriting(292934)
    training_set = [{'id': 1337,
                     'is_in_testset': 0,
                     'formula_id': 42,
                     'handwriting': sample,
                     'formula_in_latex': 'B'}]
    mult_queue = [data_multiplication.Multiply()]
    create_pfiles.training_set_multiplication(training_set, mult_queue)
    # nose.tools.assert_equal(len(feature_list), len(correct))


def execution_test():
    formula_id2index = {1337: 1, 12: 2}
    feature_folder = '.'
    index2latex = {1: '\\alpha', 2: '\\beta'}
    create_pfiles._create_index_formula_lookup(formula_id2index,
                                               feature_folder,
                                               index2latex)


def parser_test():
    create_pfiles.get_parser()


def prepare_dataset_test():
    dataset = []
    for i in range(200):
        dataset.append({'handwriting': th.get_symbol_as_handwriting(97705),
                        'formula_id': 42})
    # dataset[-1]['handwriting'].formula_id = 42
    formula_id2index = {}
    formula_id2index[42] = 1
    feature_list = [features.StrokeCount()]
    is_traindata = False
    create_pfiles.prepare_dataset(dataset,
                                  formula_id2index,
                                  feature_list,
                                  is_traindata)


def normalize_features_one_test():
    feature_list = [features.Width(), features.Height()]
    prepared = [([123], 1)]
    is_traindata = True
    out = create_pfiles._normalize_features(feature_list,
                                            prepared,
                                            is_traindata)
    nose.tools.assert_equal(out, [([0.0], 1)])


def normalize_features_two_test():
    feature_list = [features.Width(), features.Height()]
    prepared = [([123], 1), ([100], 1)]
    is_traindata = True
    out = create_pfiles._normalize_features(feature_list,
                                            prepared,
                                            is_traindata)
    # Mean: 111.5; Range: 23
    nose.tools.assert_equal(out, [([0.5], 1), ([-0.5], 1)])

    # Now the other set
    prepared = [([111.5], 1), ([90], 1), ([180], 1)]
    is_traindata = False
    out = create_pfiles._normalize_features(feature_list,
                                            prepared,
                                            is_traindata)
    nose.tools.assert_equal(out, [([0.0], 1),
                                  ([-0.93478260869565222], 1),
                                  ([2.9782608695652173], 1)])


def normalize_features_two_feats_test():
    feature_list = [features.Width(), features.Height()]
    prepared = [([123, 123], 1), ([100, 100], 1)]
    is_traindata = True
    out = create_pfiles._normalize_features(feature_list,
                                            prepared,
                                            is_traindata)
    # Mean: 111.5; Range: 23
    nose.tools.assert_equal(out, [([0.5, 0.5], 1), ([-0.5, -0.5], 1)])

    # Now the other set
    prepared = [([111.5, 111.5], 1), ([146, 146], 1), ([54, 54], 1)]
    is_traindata = False
    out = create_pfiles._normalize_features(feature_list,
                                            prepared,
                                            is_traindata)
    nose.tools.assert_equal(out, [([0.0, 0.0], 1),
                                  ([1.5, 1.5], 1),
                                  ([-2.5, -2.5], 1)])


def normalize_features_two_feats2_test():
    feature_list = [features.Width(), features.Height()]
    prepared = [([123, 123], 1), ([100, 100], 1)]
    is_traindata = True
    out = create_pfiles._normalize_features(feature_list,
                                            prepared,
                                            is_traindata)
    # Mean: 111.5; Range: 23
    nose.tools.assert_equal(out, [([0.5, 0.5], 1), ([-0.5, -0.5], 1)])

    # Now the other set
    prepared = [([111.5, 146], 1), ([146, 111.5], 1), ([54, 54], 1)]
    is_traindata = False
    out = create_pfiles._normalize_features(feature_list,
                                            prepared,
                                            is_traindata)
    nose.tools.assert_equal(out, [([0.0, 1.5], 1),
                                  ([1.5, 0.0], 1),
                                  ([-2.5, -2.5], 1)])


def normalize_features_two_classes_test():
    feature_list = [features.Width(), features.Height()]
    prepared = [([123], 1), ([100], 1), ([500], 2)]
    is_traindata = True
    out = create_pfiles._normalize_features(feature_list,
                                            prepared,
                                            is_traindata)
    # Mean: 241; Range: 400
    nose.tools.assert_equal(out, [([-0.295], 1),
                                  ([-0.3525], 1),
                                  ([0.6475], 2)])


def create_translation_file_test():
    feature_folder = os.path.join(utils.get_project_root(),
                                  "feature-files",
                                  "small-baseline")
    dataset_name = "testtestdata"
    translation = [(133700, '\\alpha', 42)]
    formula_id2index = {42: 1}
    create_pfiles._create_translation_file(feature_folder,
                                           dataset_name,
                                           translation,
                                           formula_id2index)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Segmentation is the task of splitting a data sequence into chuncks which can
then be classified chunk by chunk.

In the case of handwritten formula recognition it might be ok to segment by
strokes. However, one should note that cursive handwriting might make it
necessary to split on the point level.

Segmenting in the order in which strokes were written is also problematic as
delayed strokes (e.g. for extending a fraction stroke which was too short)
might occur.

This module contains algorithms for segmentation.
"""

import logging
import json

import itertools
import numpy
import os

import pickle
import pkg_resources

import time

import math
import scipy.sparse.csgraph

# hwrt modules
# from . import HandwrittenData
from hwrt import utils
from hwrt.HandwrittenData import HandwrittenData
from hwrt import features
from hwrt import geometry
from hwrt import partitions


def _get_symbol_index(stroke_id_needle, segmentation):
    """
    Parameters
    ----------
    stroke_id_needle : int
        Identifier for the stroke of which the symbol should get found.
    segmentation : list of lists of integers
        An ordered segmentation of strokes to symbols.

    Returns
    -------
    The symbol index in which stroke_id_needle occurs

    Examples
    --------
    >>> _get_symbol_index(3, [[0, 1, 2], [3, 4, 5], [6, 7]])
    1
    >>> _get_symbol_index(6, [[0, 1, 2], [3, 4, 5], [6, 7]])
    2
    >>> _get_symbol_index(7, [[0, 1, 2], [3, 4, 5], [6, 7]])
    2
    """
    for symbol_index, symbol in enumerate(segmentation):
        if stroke_id_needle in symbol:
            return symbol_index
    return None


def get_segmented_raw_data():
    import pymysql.cursors
    cfg = utils.get_database_configuration()
    mysql = cfg['mysql_online']
    connection = pymysql.connect(host=mysql['host'],
                                 user=mysql['user'],
                                 passwd=mysql['passwd'],
                                 db=mysql['db'],
                                 cursorclass=pymysql.cursors.DictCursor)
    cursor = connection.cursor()
    sql = ("SELECT `id`, `data`, `segmentation` "
           "FROM `wm_raw_draw_data` WHERE `segmentation` "
           "IS NOT NULL AND `wild_point_count` = 0 "
           "ORDER BY `id` LIMIT 0, 4000")
    cursor.execute(sql)
    datasets = cursor.fetchall()
    return datasets


def filter_recordings(recordings):
    new_recordings = []
    for recording in recordings:
        recording['data'] = json.loads(recording['data'])
        recording['segmentation'] = normalize_segmentation(json.loads(recording['segmentation']))
        had_none = False
        for stroke in recording['data']:
            for point in stroke:
                if point['time'] is None:
                    logging.debug("Had None-time: %i", recording['id'])
                    had_none = True
                    break
            if had_none:
                break
        if not had_none:
            new_recordings.append(recording)

    recordings = new_recordings
    logging.info("Done filtering")
    return recordings


def get_dataset():
    """Create a dataset for machine learning.

    Returns
    -------
    tuple :
        (X, y) where X is a list of tuples. Each tuple is a feature. y
        is a list of labels (0 for 'not in one symbol' and 1 for 'in symbol')
    """
    seg_data = "segmentation-X.npy"
    seg_labels = "segmentation-y.npy"
    #seg_ids = "segmentation-ids.npy"
    if os.path.isfile(seg_data) and os.path.isfile(seg_labels):
        X = numpy.load(seg_data)
        y = numpy.load(seg_labels)
        with open('datasets.pickle', 'rb') as f:
            datasets = pickle.load(f)
        return (X, y, datasets)
    datasets = get_segmented_raw_data()
    X, y = [], []
    for i, data in enumerate(datasets):
        if i % 10 == 0:
            logging.info("i=%i", i)
        # logging.info("Start looking at dataset %i", i)
        segmentation = json.loads(data['segmentation'])
        # logging.info(segmentation)
        recording = json.loads(data['data'])
        X_symbol = [get_median_stroke_distance(recording)]
        if len([p for s in recording for p in s if p['time'] is None]) > 0:
            continue
        for strokeid1, strokeid2 in itertools.combinations(list(range(len(recording))), 2):
            stroke1 = recording[strokeid1]
            stroke2 = recording[strokeid2]
            if len(stroke1) == 0 or len(stroke2) == 0:
                logging.debug("stroke len 0. Skip.")
                continue
            X.append(get_stroke_features(recording, strokeid1, strokeid2)+X_symbol)
            same_symbol = (_get_symbol_index(strokeid1, segmentation) ==
                           _get_symbol_index(strokeid2, segmentation))
            y.append(int(same_symbol))
    X = numpy.array(X, dtype=numpy.float32)
    y = numpy.array(y, dtype=numpy.int32)
    numpy.save(seg_data, X)
    numpy.save(seg_labels, y)
    datasets = filter_recordings(datasets)
    with open('datasets.pickle', 'wb') as f:
        pickle.dump(datasets, f, protocol=pickle.HIGHEST_PROTOCOL)
    return (X, y, datasets)


def get_nn_classifier(X, y):
    import lasagne
    import theano
    import theano.tensor as T
    N_CLASSES = 2

    # First, construct an input layer.
    # The shape parameter defines the expected input shape, which is just the
    # shape of our data matrix X.
    l_in = lasagne.layers.InputLayer(shape=X.shape)
    # A dense layer implements a linear mix (xW + b) followed by a nonlinear
    # function.
    hiddens = [64, 64, 64]  # sollte besser als 0.12 sein (mit [32])
    layers = [l_in]

    for n_units in hiddens:
        l_hidden_1 = lasagne.layers.DenseLayer(
            layers[-1],  # The first argument is the input to this layer
            num_units=n_units,  # the layer's output dimensionality
            nonlinearity=lasagne.nonlinearities.tanh)
        layers.append(l_hidden_1)
    # For our output layer, we'll use a dense layer with a softmax nonlinearity.
    l_output = lasagne.layers.DenseLayer(layers[-1], num_units=N_CLASSES,
                                         nonlinearity=lasagne.nonlinearities.softmax)
    # Now, we can generate the symbolic expression of the network's output
    # given an input variable.
    net_input = T.matrix('net_input')
    net_output = l_output.get_output(net_input)
    # As a loss function, we'll use Theano's categorical_crossentropy function.
    # This allows for the network output to be class probabilities,
    # but the target output to be class labels.
    true_output = T.ivector('true_output')
    loss = T.mean(T.nnet.categorical_crossentropy(net_output, true_output))

    reg = lasagne.regularization.l2(l_output)
    loss = loss + 0.001*reg
    #NLL_LOSS = -T.sum(T.log(p_y_given_x)[T.arange(y.shape[0]), y]) Retrieving
    # all parameters of the network is done using get_all_params, which
    # recursively collects the parameters of all layers connected to the
    # provided layer.
    all_params = lasagne.layers.get_all_params(l_output)

    # Now, we'll generate updates using Lasagne's SGD function
    updates = lasagne.updates.momentum(loss, all_params, learning_rate=0.1)

    # Finally, we can compile Theano functions for training and computing the
    # output.
    train = theano.function([net_input, true_output], loss, updates=updates)
    get_output = theano.function([net_input], net_output)

    logging.debug("|X|=%i", len(X))
    logging.debug("|y|=%i", len(y))
    logging.debug("|X[0]|=%i", len(X[0]))

    # Train
    epochs = 20
    for n in range(epochs):
        train(X, y)
    return get_output


def get_stroke_features(recording, strokeid1, strokeid2):
    """Get the features used to decide if two strokes belong to the same symbol
    or not.

    * Distance of bounding boxes
    """
    stroke1 = recording[strokeid1]
    stroke2 = recording[strokeid2]
    assert isinstance(stroke1, list), "stroke1 is a %s" % type(stroke1)
    X_i = []
    for s in [stroke1, stroke2]:
        hw = HandwrittenData(json.dumps([s]))
        feat1 = features.ConstantPointCoordinates(strokes=1,
                                                  points_per_stroke=20,
                                                  fill_empty_with=0)
        feat2 = features.ReCurvature(strokes=1)
        feat3 = features.Ink()
        X_i += hw.feature_extraction([feat1, feat2, feat3])
    X_i += [get_strokes_distance(stroke1, stroke2)]  # Distance of strokes
    X_i += [get_time_distance(stroke1, stroke2)]  # Time in between
    X_i += [abs(strokeid2-strokeid1)]  # Strokes in between
    return X_i


def get_median_stroke_distance(recording):
    dists = []
    for s1_id in range(len(recording)-1):
        for s2_id in range(s1_id+1, len(recording)):
            dists.append(get_strokes_distance(recording[s1_id],
                                              recording[s2_id]))
    return numpy.median(dists)


def get_time_distance(s1, s2):
    min_dist = abs(s1[0]['time'] - s2[0]['time'])
    for p1, p2 in zip(s1, s2):
        dist = abs(p1['time'] - p2['time'])
        min_dist = min(min_dist, dist)
    return min_dist


def get_strokes_distance(s1, s2):
    if len(s1) == 1:
        s1 += s1
    if len(s2) == 1:
        s2 += s2
    stroke1 = geometry.PolygonalChain(s1)
    stroke2 = geometry.PolygonalChain(s2)

    min_dist = geometry.segments_distance(stroke1[0], stroke2[0])
    for seg1, seg2 in itertools.product(stroke1, stroke2):
        min_dist = min(min_dist, geometry.segments_distance(seg1, seg2))
    return min_dist


def merge_segmentations(segs1, segs2, strokes=None):
    """
    Parameters
    ----------
    segs1 : a list of tuples
        Each tuple is a segmentation with its score
    segs2 : a list of tuples
        Each tuple is a segmentation with its score
    strokes : list of stroke names for segs2

    Returns
    -------
    list of tuples :
        Segmentations with their score, combined from segs1 and segs2
    """
    def translate(segmentation, strokes):
        t = []
        for symbol in segmentation:
            symbol_new = []
            for stroke in symbol:
                symbol_new.append(strokes[stroke])
            t.append(symbol_new)
        return t
    if strokes is None:
        strokes = [i for i in range(len(segs2[0][0]))]
    topf = partitions.TopFinder(500)
    for s1, s2 in itertools.product(segs1, segs2):
        topf.push(s1[0]+translate(s2[0], strokes), s1[1]*s2[1])
    return list(topf)


def update_segmentation_data(segmentation, add):
    return [[el + add for el in symbol] for symbol in segmentation]


def get_segmentation(recording,
                     single_clf,
                     single_stroke_clf,
                     stroke_segmented_classifier):
    """Get a list of segmentations of recording with the probability of the
    segmentation being correct.

    Parameters
    ----------
    recording : A list of lists
        Each sublist represents a stroke
    single_clf : object
        A classifier for single symbols
    single_stroke_clf : object
        A classifier which decides if a single stroke is a complete symbol
    stroke_segmented_classifier : object
        Classifier which decides if two strokes belong to one symbol or not

    Returns
    -------
    list of tuples :
        Segmentations together with their probabilities. Each probability
        has to be positive and the sum may not be bigger than 1.0.

    Examples
    --------
    >>> stroke1 = [{'x': 0, 'y': 0, 'time': 0}, {'x': 12, 'y': 12, 'time': 1}]
    >>> stroke2 = [{'x': 0, 'y': 10, 'time': 2}, {'x': 12, 'y': 0, 'time': 3}]
    >>> stroke3 = [{'x': 14, 'y': 0, 'time': 5}, {'x': 14, 'y': 12, 'time': 6}]
    >>> #get_segmentation([stroke1, stroke2, stroke3], single_clf)
    [
      ([[0, 1], [2]], 0.8),
      ([[0], [1,2]], 0.1),
      ([[0,2], [1]], 0.05)
    ]
    """

    # TODO: Use it
    points = get_points(recording)
    mst = get_mst(points)
    mst_wood = [{'mst': mst, 'strokes': list(range(len(mst)))}]
    # TODO: break mst into wood of msts wherever possible by recognizing single
    # symbols by stroke

    bbintersections = get_bb_intersections(recording)
    for i, stroke in enumerate(recording):  # TODO
        predictions = single_clf.predict({'id': 0, 'data': [stroke]})
        prob_sum = sum([p['probability'] for p in predictions[:3]])
        # dots cannot be segmented into single symbols at this point
        if prob_sum > 0.95 and not any([el for el in bbintersections[i]]) and len(stroke) > 2 and predictions[0]['semantics'].split(';')[1] != '-':
            # Split mst here
            split_mst_index, split_node_i = find_split_node(mst_wood, i)
            mst_wood_tmp = break_mst(mst_wood[split_mst_index],
                                     split_node_i)
            del mst_wood[split_mst_index]
            for mst in mst_wood_tmp:
                mst_wood.append(mst)
            for mst in mst_wood:
                if i in mst['strokes']:
                    mst['pred'] = predictions[0]['semantics'].split(';')[1]

    # if any([True for mst in mst_wood if len(mst['strokes']) >= 8]):
    #     logging.debug([mst['pred'] for mst in mst_wood if 'pred' in mst])
    #     HandwrittenData(json.dumps(recording)).show()
    return [(normalize_segmentation([mst['strokes'] for mst in mst_wood]), 1.0)]

    # HandwrittenData(json.dumps(recording)).show()
    # return [([[i for i in range(len(recording))]], 1.0)]
    ##mst_wood = break_mst(mst, recording)  # TODO

    # for i in range(0, 2**len(points)):
    #     segmentation = get_segmentation_from_mst(mst, i)
    # TODO

    X_symbol = [get_median_stroke_distance(recording)]

    # Pre-segment to 8 strokes
    # TODO: Take first 4 strokes and add strokes within their bounding box
    # TODO: What if that is more then 8 strokes?
    # -> Geometry
    #    Build tree structure. A stroke `c` is the child of another stroke `p`,
    #    if the bounding box of `c` is within the bounding box of `p`.
    #       Problem: B <-> 13
    top_segmentations_global = [([], 1.0)]
    for chunk_part in mst_wood:  # range(int(math.ceil(float(len(recording))/8))):
        #chunk = recording[8*chunk_part:8*(chunk_part+1)]
        chunk = [recording[stroke] for stroke in chunk_part['strokes']]

        # Segment after pre-segmentation
        prob = [[1.0 for _ in chunk] for _ in chunk]
        for strokeid1 in range(len(chunk)):
            for strokeid2 in range(len(chunk)):
                if strokeid1 == strokeid2:
                    continue
                X = get_stroke_features(chunk, strokeid1, strokeid2)
                X += X_symbol
                X = numpy.array([X], dtype=numpy.float32)
                prob[strokeid1][strokeid2] = stroke_segmented_classifier(X)

        top_segmentations = list(partitions.get_top_segmentations(prob, 500))
        for i, segmentation in enumerate(top_segmentations):
            symbols = apply_segmentation(chunk, segmentation)
            min_top2 = partitions.TopFinder(1, find_min=True)
            for i, symbol in enumerate(symbols):
                predictions = single_clf.predict(symbol)
                min_top2.push("value-%i" % i,
                              predictions[0]['probability'] + predictions[1]['probability'])
            top_segmentations[i][1] *= list(min_top2)[0][1]
        # for i, segmentation in enumerate(top_segmentations):
        #     top_segmentations[i][0] = update_segmentation_data(top_segmentations[i][0], 8*chunk_part)
        top_segmentations_global = merge_segmentations(top_segmentations_global, top_segmentations, chunk_part['strokes'])
    return [(normalize_segmentation(seg), prob) for seg, prob in top_segmentations_global]


def has_missing_break(real_seg, pred_seg):
    """
    Parameters
    ----------
    real_seg : list of integers
        The segmentation as it should be.
    pred_seg : list of integers
        The predicted segmentation.

    Returns
    -------
    bool :
        True, if strokes of two different symbols are put in the same symbol.
    """
    for symbol_pred in pred_seg:
        for symbol_real in real_seg:
            if symbol_pred[0] in symbol_real:
                for stroke in symbol_pred:
                    if stroke not in symbol_real:
                        return True
    return False


def has_wrong_break(real_seg, pred_seg):
    """
    Parameters
    ----------
    real_seg : list of integers
        The segmentation as it should be.
    pred_seg : list of integers
        The predicted segmentation.

    Returns
    -------
    bool :
        True, if strokes of one symbol were segmented to be in different
        symbols.
    """
    for symbol_real in real_seg:
        for symbol_pred in pred_seg:
            if symbol_real[0] in symbol_pred:
                for stroke in symbol_real:
                    if stroke not in symbol_pred:
                        return True
    return False


def find_split_node(mst_wood, i):
    """
    Parameters
    ----------
    mst_wood : list of dictionarys
    i : int
        Number of the stroke where one mst gets split

    Returns
    -------
    tuple :
        (mst index, node index)
    """
    for mst_index, mst in enumerate(mst_wood):
        if i in mst['strokes']:
            return (mst_index, mst['strokes'].index(i))
    raise ValueError('%i was not found as stroke index.' % i)


def break_mst(mst, i):
    """Break mst into multiple MSTs by removing one node i.

    Parameters
    ----------
    mst : symmetrical square matrix
    i : index of the mst where to break

    Returns
    -------
    list of dictionarys ('mst' and 'strokes' are the keys)
    """
    for j in range(len(mst['mst'])):
        mst['mst'][i][j] = 0
        mst['mst'][j][i] = 0
    _, components = scipy.sparse.csgraph.connected_components(mst['mst'])
    comp_indices = {}
    for el in set(components):
        comp_indices[el] = {'strokes': [], 'strokes_i': []}
    for i, comp_nr in enumerate(components):
        comp_indices[comp_nr]['strokes'].append(mst['strokes'][i])
        comp_indices[comp_nr]['strokes_i'].append(i)

    mst_wood = []

    for key in comp_indices:
        matrix = []
        for i, line in enumerate(mst['mst']):
            line_add = []
            if i not in comp_indices[key]['strokes_i']:
                continue
            for j, el in enumerate(line):
                if j in comp_indices[key]['strokes_i']:
                    line_add.append(el)
            matrix.append(line_add)
        assert len(matrix) > 0, "len(matrix) == 0 (strokes: %s, mst=%s, i=%i)" % (comp_indices[key]['strokes'], mst, i)
        assert len(matrix) == len(matrix[0]), \
            "matrix was %i x %i, but should be square" % (len(matrix), len(matrix[0]))
        assert len(matrix) == len(comp_indices[key]['strokes']), \
            "stroke length was not equal to matrix length (strokes=%s, len(matrix)=%i)" % (comp_indices[key]['strokes'], len(matrix))
        mst_wood.append({'mst': matrix,
                         'strokes': comp_indices[key]['strokes']})
    return mst_wood


def _is_out_of_order(segmentation):
    """
    Check if a given segmentation is out of order.

    Examples
    --------
    >>> _is_out_of_order([[0, 1, 2, 3]])
    False
    >>> _is_out_of_order([[0, 1], [2, 3]])
    False
    >>> _is_out_of_order([[0, 1, 3], [2]])
    True
    """
    last_stroke = -1
    for symbol in segmentation:
        for stroke in symbol:
            if last_stroke > stroke:
                return True
            last_stroke = stroke
    return False


def _less_than(l, n):
    return float(len([1 for el in l if el < n]))


class single_classifier(object):
    """Classifier for single symbols."""
    def __init__(self):
        logging.info("Start reading model...")
        model_path = pkg_resources.resource_filename('hwrt', 'misc/')
        model_file = os.path.join(model_path, "model.tar")
        logging.info("Model: %s", model_file)
        (preprocessing_queue, feature_list, model,
         output_semantics) = utils.load_model(model_file)
        self.preprocessing_queue = preprocessing_queue
        self.feature_list = feature_list
        self.model = model
        self.output_semantics = output_semantics

    def predict(self, parsed_json):
        evaluate = utils.evaluate_model_single_recording_preloaded
        results = evaluate(self.preprocessing_queue,
                           self.feature_list,
                           self.model,
                           self.output_semantics,
                           json.dumps(parsed_json['data']),
                           parsed_json['id'])
        return results


class single_symbol_stroke_classifier(object):
    """Classifier which decides if a single stroke is a single symbol."""
    def __init__(self):
        logging.info("Start reading model...")
        model_path = pkg_resources.resource_filename('hwrt', 'misc/')
        model_file = os.path.join(model_path, "model-single-stroke.tar")  # TODO
        logging.info("Model: %s", model_file)
        (preprocessing_queue, feature_list, model,
         output_semantics) = utils.load_model(model_file)
        self.preprocessing_queue = preprocessing_queue
        self.feature_list = feature_list
        self.model = model
        self.output_semantics = output_semantics

    def predict(self, parsed_json):
        evaluate = utils.evaluate_model_single_recording_preloaded
        results = evaluate(self.preprocessing_queue,
                           self.feature_list,
                           self.model,
                           self.output_semantics,
                           json.dumps(parsed_json['data']),
                           parsed_json['id'])
        return results


def apply_segmentation(recording, segmentation):
    symbols = []
    seg, prob = segmentation
    for symbol_indices in seg:
        symbol = []
        for index in symbol_indices:
            symbol.append(recording[index])
        symbols.append({'data': symbol, 'id': 'symbol-%i' % index})
    return symbols


class Graph(object):
    def __init__(self):
        self.nodes = []

    def add_node(self, payload):
        """
        Returns
        -------
        int
            Identifier for the inserted node.
        """
        self.nodes.append(Node(len(self.nodes), payload))
        return len(self.nodes) - 1

    def add_edge(self, node_i, node_j):
        self.nodes[node_i].neighbors.append(self.nodes[node_j])
        self.nodes[node_j].neighbors.append(self.nodes[node_i])

    def get_connected_nodes(self):
        remaining_graph_nodes = list(range(len(self.nodes)))
        segments = []
        while len(remaining_graph_nodes) > 0:
            node_nr = remaining_graph_nodes.pop()
            segment = []
            queued = [node_nr]
            while len(queued) > 0:
                current = queued.pop()
                segment.append(current)
                remaining_graph_nodes.remove(current)
                queued = [n.identifier for n in self.nodes[current].neighbors
                          if n.identifier in remaining_graph_nodes]
            segments.append(segment)
        return segments

    def generate_euclidean_edges(self):
        n = len(self.nodes)
        self.w = numpy.zeros(shape=(n, n))
        for i in range(n):
            for j in range(n):
                self.w[i][j] = self.nodes[i].get().dist_to(self.nodes[j].get())


class Node(object):
    def __init__(self, identifier, payload):
        self.neighbors = []
        self.payload = payload
        self.identifier = identifier

    def add_neighbor(self, neighbor_node):
        self.neighbors.append(neighbor_node)

    def get(self):
        return self.payload


def get_segmentation_from_mst(mst, number):
    """Get a segmentation from a MST

    If the MST has 5 strokes and a spanning tree like
    1-\
       3-4-5
    2-/
    the number 3 (0011) would mean that the 0th edge
    and the 1st edge get cut. Lets say that the edge 0 is next to node 1 and
    edge 1 is next to node 2. Then the resulting segmentation would be
    [[1], [2], [3, 4, 5]]

    Parameters
    ----------
    mst :
        Minimum spanning tree
    number : int ({0, ..., 2^(edges in MST)-1})
        The number of the segmentation.
    """
    pass


def get_points(recording):
    """Get one point for each stroke in a recording. The point represents the
    strokes spacial position (e.g. the center of the bounding box).

    Parameters
    ----------
    recording : list of strokes

    Returns
    -------
    list :
        points
    """
    points = []
    for stroke in recording:
        point = geometry.get_bounding_box(stroke).get_center()
        points.append(point)
    return points


def get_bb_intersections(recording):
    """Get all intersections of the bounding boxes of strokes.

    Parameters
    ----------
    recording : list of lists of integers

    Returns
    -------
    A symmetrical matrix which indicates if two bounding boxes intersect.
    """
    intersections = numpy.zeros((len(recording), len(recording)),
                                dtype=bool)
    for i in range(len(recording)-1):
        a = geometry.get_bounding_box(recording[i]).grow(0.2)
        for j in range(i+1, len(recording)):
            b = geometry.get_bounding_box(recording[j]).grow(0.2)
            intersections[i][j] = geometry.do_bb_intersect(a, b)
            intersections[j][i] = intersections[i][j]
    return intersections


def get_mst(points):
    """
    Parameters
    ----------
    points : list of points (geometry.Point)
        The first element of the list is the center of the bounding box of the
        first stroke, the second one belongs to the seconds stroke, ...

    Returns
    -------
    mst : square matrix
        0 nodes the edges are not connected, > 0 means they are connected
    """
    graph = Graph()
    for point in points:
        graph.add_node(point)
    graph.generate_euclidean_edges()
    matrix = scipy.sparse.csgraph.minimum_spanning_tree(graph.w)
    mst = matrix.toarray().astype(int)
    # returned matrix is not symmetrical! make it symmetrical
    for i in range(len(mst)):
        for j in range(len(mst)):
            if mst[i][j] > 0:
                mst[j][i] = mst[i][j]
            if mst[j][i] > 0:
                mst[i][j] = mst[j][i]
    return mst


def normalize_segmentation(seg):
    return sorted([sorted(el) for el in seg])


if __name__ == '__main__':
    import sys
    logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s',
                        level=logging.DEBUG,
                        stream=sys.stdout)

    logging.info("Start doctest")
    import doctest
    doctest.testmod()

    logging.info("Get single classifier")
    single_clf = single_classifier()

    logging.info("Get stroke_segmented_classifier "
                 "(decided if two strokes are in one symbol)")
    logging.info("Start creation of training set")
    X, y, recordings = get_dataset()
    logging.info("Start training")
    nn = get_nn_classifier(X, y)
    stroke_segmented_classifier = lambda X: nn(X)[0][1]
    #import pprint
    #pp = pprint.PrettyPrinter(indent=4)
    y_predicted = numpy.argmax(nn(X), axis=1)
    classification = [yi == yip for yi, yip in zip(y, y_predicted)]
    err = float(sum([not i for i in classification]))/len(classification)
    logging.info("Error: %0.2f (for %i training examples)", err, len(y))

    logging.info("Get single stroke classifier")
    single_stroke_clf = None  # single_symbol_stroke_classifier()

    #logging.info("Get segmented raw data")
    #recordings = get_segmented_raw_data()
    logging.info("Start testing")
    score_place = []
    out_of_order_count = 0
    ## Filter recordings
    #recordings = filter_recordings(recordings)
    over_segmented_symbols = 0
    under_segmented_symbols = 0
    overunder_segmented_symbols = 0

    for nr, recording in enumerate(recordings):
        if nr % 100 == 0:
            print(("## %i " % nr) + "#"*80)

        t0 = time.time()
        seg_predict = get_segmentation(recording['data'],
                                       single_clf,
                                       single_stroke_clf,
                                       stroke_segmented_classifier)
        t1 = time.time()
        real_seg = recording['segmentation']
        if all([has_wrong_break(real_seg, seg) for seg, score in seg_predict]):
            over_segmented_symbols += 1
        if all([has_missing_break(real_seg, seg) for seg, score in seg_predict]):
            under_segmented_symbols += 1
        if all([has_wrong_break(real_seg, seg) for seg, score in seg_predict]) \
           and all([has_missing_break(real_seg, seg) for seg, score in seg_predict]):
           overunder_segmented_symbols += 1
        pred_str = ""
        for i, pred in enumerate(seg_predict):
            seg, score = pred
            if i == 0:
                pred_str = "  Predict segmentation:\t%s (%0.8f)" % (seg, score)
            #print("#{0:>3} {1:.8f}: {2}".format(i, score, seg))
            if seg == real_seg:
                score_place.append(i)
                break
        else:
            i = -1
        if all([has_wrong_break(real_seg, seg) for seg, score in seg_predict]):
            print("## %i" % recording['id'])
            print("  Real segmentation:\t%s (got at place %i)" % (real_seg, i))
            print(pred_str)
            print("  Segmentation took %0.4f seconds." % (t1-t0))
            if has_wrong_break(real_seg, seg_predict[0][0]):
                print("  over-segmented")
            if has_missing_break(real_seg, seg_predict[0][0]):
                print("  under-segmented")
        out_of_order_count += _is_out_of_order(real_seg)
    logging.info("mean: %0.2f", numpy.mean(score_place))
    logging.info("median: %0.2f", numpy.median(score_place))
    logging.info("TOP-1: %0.2f", _less_than(score_place, 1)/len(recordings))
    logging.info("TOP-3: %0.2f", _less_than(score_place, 3)/len(recordings))
    logging.info("TOP-10: %0.2f", _less_than(score_place, 10)/len(recordings))
    logging.info("TOP-20: %0.2f", _less_than(score_place, 20)/len(recordings))
    logging.info("TOP-50: %0.2f", _less_than(score_place, 50)/len(recordings))
    logging.info("Over-segmented: %0.2f", float(over_segmented_symbols)/len(recordings))
    logging.info("Under-segmented: %0.2f", float(under_segmented_symbols)/len(recordings))
    logging.info("overUnder-segmented: %0.2f", float(overunder_segmented_symbols)/len(recordings))
    logging.info("Out of order: %i", out_of_order_count)
    logging.info("Total: %i", len(recordings))



#!/usr/bin/env python

import glob
import logging
import sys
import json

logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s',
                    level=logging.DEBUG,
                    stream=sys.stdout)

import pymysql
import pymysql.cursors
import unicodedata

# hwrt modules
from hwrt import HandwrittenData
from hwrt import utils
from hwrt import datasets

replacements = [('int', r'\int'),
                ('cross', r'\times'),
                ('frac', r'\frac{}{}'),
                ('sin', r'\sin'),
                ('dx', r'\mathrm{d}x'),
                ('dt', r'\mathrm{d}t'),
                ('arcsin', r'\arcsin'),
                ('cos', r'\cos'),
                ('lim', r'\lim'),
                ('log', r'\log'),
                ('ln', r'\ln'),
                ('alpha', r'\alpha'),
                ('beta', r'\beta'),
                ('kappa', r'\kappa'),
                ('inf', r'\infty'),
                ('root', r'\sqrt{}'),
                ('pi', r'\pi'),
                ('{', '\{'),
                ('}', '\}'),
                ('leftSquareBracket', '['),
                ('rightSquareBracket', ']'),
                ('sum', r'\sum'),
                ('prod', r'\prod'),
                ('greater', r'>'),
                ('greaterEqual', r'\geq'),
                ('less', r'<'),
                ('lessEqual', r'\leq'),
                ('+-', r'\pm'),
                ('lim', r'\lim'),
                ('sin', r'\sin'),
                ('cos', r'\cos'),
                ('dot', r'\cdot'),  # really?
                ('log', r'\log'),
                ('ln', r'\ln'),
                ('rightArrow', r'\rightarrow'),
                ('verticalBar', r'|'),
                ('nabla', r'\nabla'),
                ('partial', r'\partial'),
                ('slash', r'/'),
                ('star', r'\star'),
                ('in', r'\in')]

skip = ['cotg']
# skip = ['frac', 'dt', 'dx', 'arcsin',  'tg', 'cross', "'", ',']
# '\sin', '\cos', '\lim', '\log', '\ln'


def elementtree_to_dict(element):
    """Convert an xml ElementTree to a dictionary."""
    d = dict()
    if hasattr(element, 'text') and element.text is not None:
        d['text'] = element.text

    d.update(element.items())  # element's attributes

    for c in list(element):  # element's children
        if c.tag not in d:
            d[c.tag] = elementtree_to_dict(c)
        # an element with the same tag was already in the dict
        else:
            # if it's not a list already, convert it to a list and append
            if not isinstance(d[c.tag], list):
                d[c.tag] = [d[c.tag], elementtree_to_dict(c)]
           # append to the list
            else:
                d[c.tag].append(elementtree_to_dict(c))
    return d


def strip_end(text, suffix):
    """Strip `suffix` from the end of `text` if `text` has that suffix."""
    if not text.endswith(suffix):
        return text
    return text[:len(text)-len(suffix)]


def remove_accents(input_str):
    nkfd_form = unicodedata.normalize('NFKD', input_str)
    return u"".join([c for c in nkfd_form if not unicodedata.combining(c)])


def get_recordings(directory):
    recordings = []
    for filepath in glob.glob("%s/*.xml" % directory):
        import xml.etree.ElementTree
        root = xml.etree.ElementTree.parse(filepath).getroot()
        root = elementtree_to_dict(root)

        name = root['Name']['text']

        if name in skip:
            continue

        for search, rep in replacements:
            if name == search:
                name = name.replace(search, rep)

        if name in skip:
            continue

        name = strip_end(name, '_Capital')
        examples = root['Examples']['Example']
        logging.info("Name: %s", name)

        symbol_recordings = []

        #import pprint
        #pprint.pprint(root)
        if isinstance(examples, dict):
            examples = [examples]
        for example in examples:
            recording = []
            time = 0
            if isinstance(example['strokesXml']['Strokes']['Stroke'], dict):
                example['strokesXml']['Strokes']['Stroke'] = \
                    [example['strokesXml']['Strokes']['Stroke']]
            for stroke_xml in example['strokesXml']['Strokes']['Stroke']:
                stroke = []
                # print(stroke_xml.keys())
                if isinstance(stroke_xml['Point'], dict):
                    stroke_xml['Point'] = [stroke_xml['Point']]
                for point in stroke_xml['Point']:
                    stroke.append({'x': float(point['X']),
                                   'y': float(point['Y']),
                                   'time': time})
                    time += 20
                time += 200
                recording.append(stroke)
            hw = HandwrittenData.HandwrittenData(json.dumps(recording),
                                                 formula_in_latex=name)
            info = {}
            if 'text' in example['FormulaInputInfo']['Username']:
                uname = example['FormulaInputInfo']['Username']['text'].strip()
                info['username'] = 'MfrDB::%s' % remove_accents(unicode(uname))
                info['username'] = info['username'].replace("+", "PLUS")
                info['username'] = info['username'].replace("...", "DOTS")
                info['username'] = info['username'].replace(u"\u0432\u044b\u0444", "BBEF")
                info['username'] = info['username'].replace(u"\u0437\u0438\u0438", "Zeii")
            else:
                info['username'] = 'MfrDB::unknown'
            copyright_str = ("This dataset was contributed by MfrDB. You can "
                             "download their complete dataset at "
                             "[mfr.felk.cvut.cz/Database.html]"
                             "(http://mfr.felk.cvut.cz/Database.html)")
            info['userid'] = datasets.getuserid(info['username'],
                                                copyright_str)
            import uuid
            info['secret'] = str(uuid.uuid4())
            info['ip'] = example['FormulaInputInfo']['Address']['text']
            import IPy
            info['ip'] = IPy.IP(info['ip']).int()
            info['accepted_formula_id'] = datasets.formula_to_dbid(name)
            info['client'] = example['FormulaInputInfo']['Client']['text']
            from dateutil.parser import parse
            info['creation_date'] = parse(example['FormulaInputInfo']['Time']['text'])
            info['device_type'] = example['FormulaInputInfo']['Device']['text'].lower()
            info['sample_id'] = example['FormulaInputInfo']['SampleId']['text']
            info['rec_desc'] = "%s::%s::%s::%s::%s" % (filepath,
                                                       example['Id'],
                                                       info['sample_id'],
                                                       info['client'],
                                                       example['FormulaInputInfo']['Address']['text'])
            info['description'] = ("This dataset was contributed by MfrDB. "
                                   "You can download their complete dataset "
                                   "at [mfr.felk.cvut.cz/Database.html](http://mfr.felk.cvut.cz/Database.html)")
            symbol_recordings.append((hw, info))
        recordings.append((name, symbol_recordings))
    return recordings



"""Utility functions to work with other datasets."""

import logging
import sys

logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s',
                    level=logging.DEBUG,
                    stream=sys.stdout)

import pymysql
import pymysql.cursors

# hwrt modules
from hwrt import utils

__formula_to_dbid_cache = None
username2id = {}


def formula_to_dbid(formula_str):
    """Convert a LaTeX formula to the database index.

    Parameters
    ----------
    formula_str : string
        The formula as LaTeX code.

    Returns
    -------
    int :
        The database index.
    """
    global __formula_to_dbid_cache
    if __formula_to_dbid_cache is None:
        cfg = utils.get_database_configuration()
        mysql = cfg['mysql_dev']
        connection = pymysql.connect(host=mysql['host'],
                                     user=mysql['user'],
                                     passwd=mysql['passwd'],
                                     db=mysql['db'],
                                     charset='utf8mb4',
                                     cursorclass=pymysql.cursors.DictCursor)
        cursor = connection.cursor()

        # Get all formulas that should get examined
        sql = ("SELECT `id`, `formula_in_latex` FROM `wm_formula` ")
        cursor.execute(sql)
        formulas = cursor.fetchall()
        __formula_to_dbid_cache = {}
        for fm in formulas:
            __formula_to_dbid_cache[fm['formula_in_latex']] = fm['id']
    if formula_str in __formula_to_dbid_cache:
        return __formula_to_dbid_cache[formula_str]
    else:
        cfg = utils.get_database_configuration()
        mysql = cfg['mysql_dev']
        connection = pymysql.connect(host=mysql['host'],
                                     user=mysql['user'],
                                     passwd=mysql['passwd'],
                                     db=mysql['db'],
                                     charset='utf8mb4',
                                     cursorclass=pymysql.cursors.DictCursor)
        cursor = connection.cursor()
        sql = ("INSERT INTO `wm_formula` (`user_id`, `formula_name`, "
               "`formula_in_latex`, "
               "`mode`, `package`, "
               "`is_important`) VALUES ("
               "'10', %s, %s, 'bothmodes', NULL, '0');")
        logging.info("Insert %s.", formula_str)
        cursor.execute(sql, (formula_str, formula_str))
        connection.commit()
        __formula_to_dbid_cache[formula_str] = connection.insert_id()
        return __formula_to_dbid_cache[formula_str]


def getuserid(username, copyright_str):
    """Get the ID of the user with `username` from write-math.com. If he
    doesn't exist by now, create it. Add `copyright_str` as a description.

    Parameters
    ----------
    username : string
        Name of a user.
    copyright_str : string
        Description text of a user in Markdown format.

    Returns
    -------
    int :
        ID on write-math.com of the user.
    """
    global username2id
    if username not in username2id:
        cfg = utils.get_database_configuration()
        mysql = cfg['mysql_dev']
        connection = pymysql.connect(host=mysql['host'],
                                     user=mysql['user'],
                                     passwd=mysql['passwd'],
                                     db=mysql['db'],
                                     charset='utf8mb4',
                                     cursorclass=pymysql.cursors.DictCursor)
        cursor = connection.cursor()

        sql = ("INSERT IGNORE INTO  `wm_users` ("
               "`display_name` , "
               "`password` ,"
               "`account_type` ,"
               "`confirmation_code` ,"
               "`status` ,"
               "`description`"
               ") "
               "VALUES ("
               "%s, '',  'Regular User',  '',  'activated', %s"
               ");")
        cursor.execute(sql, (username, copyright_str))
        connection.commit()

        # Get the id
        try:
            sql = ("SELECT  `id` FROM  `wm_users` "
                   "WHERE  `display_name` =  %s LIMIT 1")
            cursor.execute(sql, username)
            uid = cursor.fetchone()['id']
        except Exception as inst:
            logging.debug("username not found: %s", username)
            print(inst)
        # logging.info("%s: %s", username, uid)
        username2id[username] = uid
    return username2id[username]


def insert_recording(hw):
    """Insert recording `hw` into database."""
    cfg = utils.get_database_configuration()
    mysql = cfg['mysql_dev']
    connection = pymysql.connect(host=mysql['host'],
                                 user=mysql['user'],
                                 passwd=mysql['passwd'],
                                 db=mysql['db'],
                                 charset='utf8mb4',
                                 cursorclass=pymysql.cursors.DictCursor)
    try:
        cursor = connection.cursor()
        sql = ("INSERT INTO `wm_raw_draw_data` ("
               "`user_id`, "
               "`data`, "
               "`md5data`, "
               "`creation_date`, "
               "`device_type`, "
               "`accepted_formula_id`, "
               "`secret`, "
               "`ip`, "
               "`segmentation`, "
               "`internal_id`, "
               "`description` "
               ") VALUES (%s, %s, MD5(data), "
               "%s, %s, %s, %s, %s, %s, %s, %s);")
        data = (hw.user_id,
                hw.raw_data_json,
                getattr(hw, 'creation_date', None),
                getattr(hw, 'device_type', ''),
                getattr(hw, 'accepted_formula_id', None),
                getattr(hw, 'secret', ''),
                getattr(hw, 'ip', None),
                str(getattr(hw, 'segmentation', '')),
                getattr(hw, 'internal_id', ''),
                getattr(hw, 'description', ''))
        cursor.execute(sql, data)
        connection.commit()
        for symbol_id, strokes in zip(hw.symbol_stream, hw.segmentation):
            insert_symbol_mapping(cursor.lastrowid, symbol_id, hw.user_id, strokes)
        logging.info("Insert raw data.")
    except pymysql.err.IntegrityError as e:
        print("Error: {} (can probably be ignored)".format(e))


def insert_symbol_mapping(raw_data_id, symbol_id, user_id, strokes):
    """
    Insert data into `wm_strokes_to_symbol`.

    Parameters
    ----------
    raw_data_id : int
    user_id : int
    strokes: list of int
    """
    cfg = utils.get_database_configuration()
    mysql = cfg['mysql_dev']
    connection = pymysql.connect(host=mysql['host'],
                                 user=mysql['user'],
                                 passwd=mysql['passwd'],
                                 db=mysql['db'],
                                 charset='utf8mb4',
                                 cursorclass=pymysql.cursors.DictCursor)
    cursor = connection.cursor()
    sql = ("INSERT INTO `writemath`.`wm_strokes_to_symbol` "
           "(`raw_data_id`, `symbol_id`, `strokes`, `user_id`) "
           "VALUES (%s, %s, %s, %s);")
    data = (raw_data_id,
            symbol_id,
            ",".join([str(stroke) for stroke in strokes]),
            user_id)
    cursor.execute(sql, data)
    connection.commit()



#!/usr/bin/env python

import json
import signal
import sys
import logging
logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s',
                    level=logging.DEBUG,
                    stream=sys.stdout)

from natsort import natsorted
from xml.dom.minidom import parseString

# hwrt modules
from hwrt import HandwrittenData
from hwrt.datasets import formula_to_dbid


def beautify_xml(path):
    """Beautify / pretty print XML in `path`."""
    with open(path) as f:
        content = f.read()

    pretty_print = lambda data: '\n'.join([line for line in parseString(data).toprettyxml(indent=' '*2).split('\n') if line.strip()])
    return pretty_print(content)


def normalize_symbol_name(symbol_name):
    if symbol_name == '\\frac':
        return '\\frac{}{}'
    elif symbol_name == '\\sqrt':
        return '\\sqrt{}'
    elif symbol_name == '&lt;':
        return '<'
    elif symbol_name == '&gt;':
        return '>'
    elif symbol_name == '{':
        return '\{'
    elif symbol_name == '}':
        return '\}'
    return symbol_name


def read(filepath):
    """
    Read a single InkML file

    Parameters
    ----------
    filepath : string
        path to the (readable) InkML file

    Returns
    -------
    HandwrittenData :
        The parsed InkML file as a HandwrittenData object
    """
    import xml.etree.ElementTree
    root = xml.etree.ElementTree.parse(filepath).getroot()

    # Get the raw data
    recording = []
    strokes = sorted(root.findall('{http://www.w3.org/2003/InkML}trace'),
                     key=lambda child: child.attrib['id'])
    time = 0
    for stroke in strokes:
        stroke = stroke.text.strip().split(',')
        stroke = [point.strip().split(' ') for point in stroke]
        if len(stroke[0]) == 3:
            stroke = [{'x': float(x), 'y': float(y), 'time': float(t)}
                      for x, y, t in stroke]
        else:
            stroke = [{'x': float(x), 'y': float(y)} for x, y in stroke]
            new_stroke = []
            for p in stroke:
                new_stroke.append({'x': p['x'], 'y': p['y'], 'time': time})
                time += 20
            stroke = new_stroke
            time += 200
        recording.append(stroke)

    # Get LaTeX
    formula_in_latex = None
    # TODO: Make sure the order is correct!!!!!
    annotations = root.findall('{http://www.w3.org/2003/InkML}annotation')
    for annotation in annotations:
        if annotation.attrib['type'] == 'truth':
            formula_in_latex = annotation.text
    hw = HandwrittenData.HandwrittenData(json.dumps(recording),
                                         formula_in_latex=formula_in_latex)
    for annotation in annotations:
        if annotation.attrib['type'] == 'writer':
            hw.writer = annotation.text
        elif annotation.attrib['type'] == 'category':
            hw.category = annotation.text
        elif annotation.attrib['type'] == 'expression':
            hw.expression = annotation.text

    # Get segmentation
    segmentation = []
    # TODO: Make sure the order is correct!!!!!
    traceGroups = root.findall('{http://www.w3.org/2003/InkML}traceGroup')
    if len(traceGroups) != 1:
        raise Exception('Malformed InkML',
                        'Exactly 1 top level traceGroup expected, found %i.' %
                        len(traceGroups))
    traceGroup = traceGroups[0]
    symbol_stream = []  # has to be consistent with segmentation
    # TODO: Make sure the order is correct!!!!!
    for tg in traceGroup.findall('{http://www.w3.org/2003/InkML}traceGroup'):
        annotations = tg.findall('{http://www.w3.org/2003/InkML}annotation')
        if len(annotations) != 1:
            raise ValueError("%i annotations found for '%s'." %
                             (len(annotations), filepath))
        symbol_stream.append(formula_to_dbid(normalize_symbol_name(annotations[0].text)))
        traceViews = tg.findall('{http://www.w3.org/2003/InkML}traceView')
        symbol = []
        for traceView in traceViews:
            symbol.append(int(traceView.attrib['traceDataRef']))
        segmentation.append(symbol)
    hw.symbol_stream = symbol_stream
    hw.segmentation = segmentation
    _flat_seg = [stroke for symbol in segmentation for stroke in symbol]
    assert len(_flat_seg) == len(recording), \
        ("Segmentation had length %i, but recording has %i strokes" %
         (len(_flat_seg), len(recording)))
    assert set(_flat_seg) == set(range(len(_flat_seg)))
    hw.inkml = beautify_xml(filepath)
    hw.filepath = filepath
    return hw


def read_folder(folder):
    """
    Parameters
    ----------
    folder : string
        Path to a folde with *.inkml files.

    Returns
    -------
    list :
        Objects of the type HandwrittenData
    """
    import glob
    recordings = []
    for filename in natsorted(glob.glob("%s/*.inkml" % folder)):
        #logging.info(filename)
        hw = read(filename)
        if hw.formula_in_latex is not None:
            hw.formula_in_latex = hw.formula_in_latex.strip()
        if hw.formula_in_latex is None or not hw.formula_in_latex.startswith('$') or not hw.formula_in_latex.endswith('$'):
            if hw.formula_in_latex is not None:
                logging.info("Starts with: %s", str(hw.formula_in_latex.startswith('$')))
                logging.info("ends with: %s", str(hw.formula_in_latex.endswith('$')))
            logging.info(hw.formula_in_latex)
            logging.info(hw.segmentation)
            hw.show()
        recordings.append(hw)
    return recordings


def main():
    #hw = read("misc/IVC_learn/f1e1.inkml")
    #hw.show()

    folder = "/home/moose/Downloads/ICFHR_package/CROHME2011_data/CROHME_training/CROHME_training/"
    logging.info(folder)
    read_folder(folder)


def handler(signum, frame):
    print('Signal handler called with signal %i' % signum)
    sys.exit(-1)

if __name__ == '__main__':
    signal.signal(signal.SIGINT, handler)
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for the utility functions."""

import nose
import mock
import os
import argparse
import pkg_resources
import json

# hwrt modules
import hwrt
import hwrt.utils as utils


# Tests
def execution_test():
    """Test if the functions execute at all."""
    utils.get_project_root()
    utils.get_latest_model(".", "model")
    utils.get_latest_working_model(".")
    utils.get_latest_successful_run(".")
    nose.tools.assert_equal(utils.get_readable_time(123), "123ms")
    nose.tools.assert_equal(utils.get_readable_time(1000*30),
                            "30s 0ms")
    nose.tools.assert_equal(utils.get_readable_time(1000*60),
                            "1 minutes 0s 0ms")
    nose.tools.assert_equal(utils.get_readable_time(1000*60*60),
                            "1h, 0 minutes 0s 0ms")
    nose.tools.assert_equal(utils.get_readable_time(2*1000*60*60),
                            "2h, 0 minutes 0s 0ms")
    nose.tools.assert_equal(utils.get_readable_time(25*1000*60*60+3),
                            "25h, 0 minutes 0s 3ms")
    utils.print_status(3, 1, 123)
    utils.get_nntoolkit()
    utils.get_database_config_file()
    utils.get_database_configuration()
    nose.tools.assert_equal(utils.sizeof_fmt(1), "1.0 bytes")
    nose.tools.assert_equal(utils.sizeof_fmt(1111), "1.1 KB")


def parser_test():
    """Check the parser."""
    from argparse import ArgumentParser
    parser = ArgumentParser()
    home = os.path.expanduser("~")
    rcfile = os.path.join(home, ".hwrtrc")
    parser.add_argument("-m", "--model",
                        dest="model",
                        type=lambda x: utils.is_valid_folder(parser, x))
    parser.parse_args(['-m', home])

    parser = ArgumentParser()
    parser.add_argument("-m", "--model",
                        dest="model",
                        type=lambda x: utils.is_valid_file(parser, x))
    parser.parse_args(['-m', rcfile])


def get_class_test():
    """The get_class function returns a class for feature and preprocessing
       algorithms.
    """
    utils.get_class('ScaleAndShift', 'preprocessing', hwrt.preprocessing)
    utils.get_class('BongaBonga', 'preprocessing', hwrt.preprocessing)


def query_yes_no_test():
    """This function is used to get a binary decision by the user.
       Sadly, there are two different ways to use it due to Python 2 / 3.
    """
    try:
        import __builtin__
        builtins_str = "__builtin__.raw_input"
    except ImportError:
        builtins_str = "builtins.input"
    with mock.patch('%s' % builtins_str, return_value='y'):
        nose.tools.assert_equal(utils.query_yes_no("bla"), True)
        nose.tools.assert_equal(utils.query_yes_no("bla", None), True)
        nose.tools.assert_equal(utils.query_yes_no("bla", "yes"), True)
        nose.tools.assert_equal(utils.query_yes_no("bla", "no"), True)
    with mock.patch('%s' % builtins_str, return_value=''):
        nose.tools.assert_equal(utils.query_yes_no("bla", 'yes'), True)
        nose.tools.assert_equal(utils.query_yes_no("bla", 'no'), False)


def input_string_test():
    """Another Python 2/3 input hack."""
    try:
        import __builtin__
        with mock.patch('__builtin__.raw_input', return_value='y'):
            nose.tools.assert_equal(utils.input_string("bla"), "y")
    except ImportError:
        with mock.patch('builtins.input', return_value='y'):
            nose.tools.assert_equal(utils.input_string("bla"), "y")


def input_int_default_test():
    """Another Python 2/3 input hack."""
    try:
        import __builtin__
        builtins_str = "__builtin__.raw_input"
    except ImportError:
        builtins_str = "builtins.input"
    with mock.patch('%s' % builtins_str, return_value='yes'):
        nose.tools.assert_equal(utils.input_int_default("bla"), 0)
        nose.tools.assert_equal(utils.input_int_default("bla", 42), 42)
    with mock.patch('%s' % builtins_str, return_value=1337):
        nose.tools.assert_equal(utils.input_int_default("bla"), 1337)
        nose.tools.assert_equal(utils.input_int_default("bla", 42), 1337)


@nose.tools.raises(Exception)
def query_yes_no_exception_test():
    """Another Python 2/3 input hack."""
    try:
        import __builtin__
        builtins_str = "__builtin__"
    except ImportError:
        import builtins
        builtins_str = "builtins"
    with mock.patch('%s.raw_input' % builtins_str, return_value=''):
        utils.query_yes_no("bla", "asdf")


@nose.tools.raises(SystemExit)
def is_valid_file_test():
    """Check if a file exists. Do this check within ArgumentParser."""
    parser = argparse.ArgumentParser()

    # Does exist
    path = os.path.realpath(__file__)
    nose.tools.assert_equal(utils.is_valid_file(parser, path), path)

    # Does not exist
    utils.is_valid_file(parser, "/etc/nonexistingfile")


@nose.tools.raises(SystemExit)
def is_valid_folder_test():
    """Similiar to is_valid_file."""
    parser = argparse.ArgumentParser()

    # Does exist
    nose.tools.assert_equal(utils.is_valid_folder(parser, "/etc"), "/etc")

    # Does not exist
    utils.is_valid_folder(parser, "/etc/nonexistingfoler")


def create_project_configuration_test():
    """Test if the creation of the project configuration works."""
    filename = 'projectconftesttmp.txt'
    utils.create_project_configuration(filename)
    if os.path.isfile(filename):
        os.remove(filename)


def get_latest_model_test():
    """Check if get_latest_model works."""
    model_folder = "/etc"
    basename = "model"
    nose.tools.assert_equal(utils.get_latest_model(model_folder, basename),
                            None)
    small = os.path.join(utils.get_project_root(),
                         "models/small-baseline")
    utils.get_latest_model(small, basename)


def choose_raw_dataset_test():
    """Check the interactive function choose_raw_dataset."""
    try:
        import __builtin__
        with mock.patch('__builtin__.raw_input', return_value=0):
            utils.choose_raw_dataset()
    except ImportError:
        with mock.patch('builtins.input', return_value=0):
            utils.choose_raw_dataset()


def get_recognizer_folders_test():
    """Test if all folders are catched."""
    small = os.path.join(utils.get_project_root(),
                         "models/small-baseline")
    folders = utils.get_recognizer_folders(small)
    wanted_folders = ['preprocessed/small-baseline',
                      'feature-files/small',
                      'models/small-baseline']
    for folder, wanted_folder in zip(folders, wanted_folders):
        nose.tools.assert_equal(folder.endswith(wanted_folder),
                                True)


def load_model_test():
    """Test if the packaged model can be loaded."""
    model_path = pkg_resources.resource_filename('hwrt', 'misc/')
    model_file = os.path.join(model_path, "model.tar")
    utils.load_model(model_file)


def evaluate_model_single_recording_preloaded_test():
    """Test if the packaged model can be used."""
    model_path = pkg_resources.resource_filename('hwrt', 'misc/')
    model_file = os.path.join(model_path, "model.tar")
    recording = [[{'x': 12, 'y': 42, 'time': 123}]]
    utils.evaluate_model_single_recording(model_file, json.dumps(recording))
    (pq, fl, model, output_semantics) = utils.load_model(model_file)

    # data = {}
    # data['recording'] = json.dumps(recording)
    # data['preprocessing_queue'] = pq
    # data['feature_list'] = fl
    # data['model'] = model
    # data['output_semantics'] = output_semantics
    # utils.evaluate_model_single_recording_preloaded(**data)



#!/usr/bin/env python

"""Tools for partitioning sets."""

# http://codereview.stackexchange.com/questions/1526/finding-all-k-subset-partitions
# http://stackoverflow.com/questions/18353280/iterator-over-all-partitions-into-k-groups
# https://docs.python.org/3/library/itertools.html
# http://stackoverflow.com/questions/9316436/how-many-different-partitions-with-exactly-n-parts-can-be-made-of-a-set-with-k-e
# https://en.wikipedia.org/wiki/Partition_of_a_set
# http://math.stackexchange.com/questions/1215983/how-can-i-get-the-maximum-score-without-iterating-all-possibilities

import logging
import sys

logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s',
                    level=logging.DEBUG,
                    stream=sys.stdout)


def prepare_table(table):
    """Make the table 'symmetric' where the lower left part of the matrix is
       the reverse probability
    """
    n = len(table)
    for i, row in enumerate(table):
        assert len(row) == n
        for j, el in enumerate(row):
            if i == j:
                table[i][i] = 0.0
            elif i > j:
                table[i][j] = 1-table[j][i]
    return table


def clusters(l, K):
    """Partition list ``l`` in ``K`` partitions.
    >>> l = [0, 1, 2]
    >>> list(clusters(l, K=3))
    [[[0], [1], [2]], [[], [0, 1], [2]], [[], [1], [0, 2]], [[0], [], [1, 2]], [[], [0], [1, 2]], [[], [], [0, 1, 2]]]
    >>> list(clusters(l, K=2))
    [[[0, 1], [2]], [[1], [0, 2]], [[0], [1, 2]], [[], [0, 1, 2]]]
    >>> list(clusters(l, K=1))
    [[[0, 1, 2]]]
    """
    if l:
        prev = None
        for t in clusters(l[1:], K):
            tup = sorted(t)
            if tup != prev:
                prev = tup
                for i in range(K):
                    yield tup[:i] + [[l[0]] + tup[i], ] + tup[i+1:]
    else:
        yield [[] for _ in range(K)]


def neclusters(l, K):
    """Partition list ``l`` in ``K`` partitions, without empty parts.

    >>> l = [0, 1, 2]
    >>> list(neclusters(l, 2))
    [[[0, 1], [2]], [[1], [0, 2]], [[0], [1, 2]]]
    >>> list(neclusters(l, 1))
    [[[0, 1, 2]]]
    """
    for c in clusters(l, K):
        if all(x for x in c):
            yield c


def all_segmentations(l):
    """Get all segmentations of a list ``l``.

    This gets bigger fast. See https://oeis.org/A000110
    For len(l) = 14 it is 190,899,322

    >>> list(all_segmentations([0, 1, 2]))
    [[[0, 1, 2]], [[0, 1], [2]], [[1], [0, 2]], [[0], [1, 2]], [[0], [1], [2]]]

    """
    for K in range(1, len(l)+1):
        gen = neclusters(l, K)
        for el in gen:
            yield el


def find_index(segmentation, stroke_id):
    """
    >>> find_index([[0, 1, 2], [3, 4], [5, 6, 7]], 0)
    0
    >>> find_index([[0, 1, 2], [3, 4], [5, 6, 7]], 1)
    0
    >>> find_index([[0, 1, 2], [3, 4], [5, 6, 7]], 5)
    2
    >>> find_index([[0, 1, 2], [3, 4], [5, 6, 7]], 6)
    2
    """
    for i, symbol in enumerate(segmentation):
        for sid in symbol:
            if sid == stroke_id:
                return i
    return -1


def q(segmentation, s1, s2):
    """Test if ``s1`` and ``s2`` are in the same symbol, given the
    ``segmentation``.
    """
    index1 = find_index(segmentation, s1)
    index2 = find_index(segmentation, s2)
    return index1 == index2


class TopFinder(object):
    """Utility datastructure to find the top n elements."""
    def __init__(self, n, find_min=False):
        self.n = n
        self.tops = []
        self.find_min = find_min

    def push(self, element, value):
        """Push an ``element`` into the datastrucutre together with its value
           and only save it if it currently is one of the top n elements.

           Drop elements if necessary.
        """
        insert_pos = 0
        for index, el in enumerate(self.tops):
            if not self.find_min and el[1] >= value:
                insert_pos = index+1
            elif self.find_min and el[1] <= value:
                insert_pos = index+1
        self.tops.insert(insert_pos, [element, value])
        self.tops = self.tops[:self.n]

    def __iter__(self):
        return self.tops.__iter__()


def score_segmentation(segmentation, table):
    """Get the score of a segmentation."""
    stroke_nr = sum(1 for symbol in segmentation for stroke in symbol)
    score = 1
    for i in range(stroke_nr):
        for j in range(i+1, stroke_nr):
            qval = q(segmentation, i, j)
            if qval:
                score *= table[i][j]
            else:
                score *= table[j][i]
    return score


def normalize_segmentation(segmentation):
    for i in range(len(segmentation)):
        segmentation[i] = sorted(segmentation[i])
    return sorted(segmentation, key=lambda x: x[0])


def get_top_segmentations(table, n):
    """
    Parameters
    ----------
    table : matrix of probabilities
        Each cell (i, j) of `table` gives the probability that i and j are in
        the same symbol.
    n : int
        Number of best segmentations which get returned
    """
    stroke_count = list(range(len(table)))
    topf = TopFinder(n)
    for curr_segmentation in all_segmentations(stroke_count):
        curr_seg_score = score_segmentation(curr_segmentation, table)
        topf.push(curr_segmentation, curr_seg_score)

    for el, score in topf:
        yield [normalize_segmentation(el), score]


def main():
    # [0,1,2], [3,4,5] [6,7]
    #            0     1     2     3     4     5     6     7
    table = [[0.00, 0.55, 0.43, 0.30, 0.28, 0.74, 0.28, 0.26],  # 0
             [0.45, 0.00, 0.67, 0.40, 0.35, 0.77, 0.30, 0.31],  # 1
             [0.57, 0.33, 0.00, 0.29, 0.28, 0.80, 0.21, 0.23],  # 2
             [0.70, 0.60, 0.71, 0.00, 0.39, 0.76, 0.29, 0.29],  # 3
             [0.72, 0.65, 0.72, 0.61, 0.00, 0.76, 0.25, 0.29],  # 4
             [0.26, 0.23, 0.20, 0.24, 0.24, 0.00, 0.30, 0.31],  # 5
             [0.72, 0.70, 0.19, 0.71, 0.75, 0.70, 0.00, 0.27],  # 6
             [0.74, 0.69, 0.77, 0.71, 0.71, 0.69, 0.73, 0.00]]  # 7
    #            0     1     2
    # table = [[0.00, 0.01, 0.99],
    #          [0.99, 0.00, 0.01],
    #          [0.01, 0.99, 0.00]]
    topfs = get_top_segmentations(table, 5)
    for el, score in topfs:
        print("%0.10f: %s" % (score, el))
    for i in range(20):
        logging.info("{0:>5}: {1:>10}".format(i, len(list(all_segmentations(list(range(i)))))))


if __name__ == '__main__':
    main()
    import doctest
    doctest.testmod()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import nose
import tests.testhelper as testhelper

# hwrt modules
from hwrt.HandwrittenData import HandwrittenData
import hwrt.features as features
import hwrt.preprocessing as preprocessing


# Tests
def features_detection_test():
    feature_queue = [{'StrokeCount': None},
                     {'ConstantPointCoordinates': [{'strokes': 4,
                                                    'points_per_stroke': 20,
                                                    'fill_empty_with': 0}]}]
    correct = [features.StrokeCount(),
               features.ConstantPointCoordinates(strokes=4,
                                                 points_per_stroke=20,
                                                 fill_empty_with=0)]
    feature_list = features.get_features(feature_queue)
    # TODO: Not only compare lengths of lists but actual contents.
    nose.tools.assert_equal(len(feature_list), len(correct))


def print_featurelist_test():
    feature_list = [features.StrokeCount(),
                    features.ConstantPointCoordinates(strokes=4,
                                                      points_per_stroke=20,
                                                      fill_empty_with=0),
                    features.ConstantPointCoordinates(strokes=0,
                                                      points_per_stroke=20,
                                                      fill_empty_with=0),
                    features.ConstantPointCoordinates(strokes=0,
                                                      points_per_stroke=20,
                                                      pen_down=False),
                    features.AspectRatio(),
                    features.Width(),
                    features.Height(),
                    features.Time(),
                    features.CenterOfMass()
                    ]
    features.print_featurelist(feature_list)


def constant_point_coordinates_test():
    f = features.ConstantPointCoordinates(strokes=0,
                                          points_per_stroke=2,
                                          pen_down=False)
    g = features.ConstantPointCoordinates(strokes=0,
                                          points_per_stroke=200,
                                          pen_down=False)
    recording = testhelper.get_symbol_as_handwriting(292934)
    f._features_without_strokes(recording)
    g._features_without_strokes(recording)
    space_evenly = preprocessing.SpaceEvenly()
    space_evenly(recording)
    f._features_without_strokes(recording)


def stroke_intersections_test():
    f = features.StrokeIntersections(strokes=4)
    recording = testhelper.get_symbol_as_handwriting(293035)
    x = f(recording)
    nose.tools.assert_equal(x, [0, 1, 0, 0, 0, 0, 0, 0, 0, 0])


def dimensionality_test():
    feature_list = [(features.StrokeCount(), 1),
                    (features.ConstantPointCoordinates(strokes=4,
                                                       points_per_stroke=20,
                                                       fill_empty_with=0),
                     160),
                    (features.ConstantPointCoordinates(strokes=0,
                                                       points_per_stroke=20,
                                                       fill_empty_with=0), 60),
                    (features.ConstantPointCoordinates(strokes=0,
                                                       points_per_stroke=20,
                                                       pen_down=False), 40),
                    (features.AspectRatio(), 1),
                    (features.Width(), 1),
                    (features.Height(), 1),
                    (features.Time(), 1),
                    (features.CenterOfMass(), 2)
                    ]
    for feat, dimension in feature_list:
        nose.tools.assert_equal(feat.get_dimension(), dimension)


def simple_execution_test():
    algorithms = [features.ConstantPointCoordinates(),
                  features.ConstantPointCoordinates(strokes=0),
                  features.FirstNPoints(),
                  # features.Bitmap(),
                  features.Ink(),
                  features.AspectRatio(),
                  features.Width(),
                  features.Time(),
                  features.CenterOfMass(),
                  features.StrokeCenter(),
                  features.StrokeCenter(8),
                  features.StrokeIntersections(),
                  features.ReCurvature()
                  ]
    for algorithm in algorithms:
        recording = testhelper.get_symbol_as_handwriting(292934)
        algorithm(recording)


def stroke_intersection1_test():
    """A '&' has one stroke. This stroke intersects itself once."""
    recording = testhelper.get_symbol_as_handwriting(97705)
    feature = features.StrokeIntersections(1)
    nose.tools.assert_equal(feature(recording), [2])


def stroke_intersection2_test():
    """A 't' has two strokes. They don't intersect themselves, but they
       intersect once."""
    recording = testhelper.get_symbol_as_handwriting(293035)
    feature = features.StrokeIntersections(2)
    nose.tools.assert_equal(feature(recording), [0, 1, 0])


def recurvature_test():
    """A 'o' ends in itself. The re-curvature is therefore 0."""
    recording = testhelper.get_symbol_as_handwriting(293036)
    feature = features.ReCurvature(1)
    feature(recording)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import nose
import shutil

import tests.testhelper as testhelper

# hwrt modules
from hwrt.HandwrittenData import HandwrittenData
import hwrt.data_analyzation_metrics as dam
import hwrt.utils as utils


# Tests
def get_metrics_test():
    d = [{'Creator': None}, {'Creator': [{'filename': 'create_test.csv'}]}]
    metrics = dam.get_metrics(d)
    nose.tools.assert_equal(len(metrics), 2)


def execution_test():
    raw_datasets = testhelper.get_raw_datasets()
    l = [dam.Creator(), dam.InstrokeSpeed(), dam.InterStrokeDistance(),
         dam.TimeBetweenPointsAndStrokes(),
         dam.AnalyzeErrors()]
    for alg in l:
        alg(raw_datasets)


def execution_test2():
    d = os.path.dirname(__file__)
    raw_datasets = os.path.join(utils.get_project_root(),
                                'raw-datasets/unittests-tiny-raw.pickle')
    shutil.copyfile(os.path.join(d, 'data/unittests-tiny-raw.pickle'),
                    raw_datasets)
    dam.TimeBetweenPointsAndStrokes(raw_datasets)
